package org.brynhild.graduation.common.parser;

import org.brynhild.graduation.common.tag.Actor;
import org.brynhild.graduation.common.tag.LogFieldDescription;
import org.brynhild.graduation.common.tag.LogFieldIgnore;
import org.brynhild.graduation.common.tag.LogGenerated;
import org.brynhild.graduation.common.utils.ClassUtil;
import org.brynhild.graduation.common.utils.StringUtil;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class LogGenerator {

    private static final String ln = "\r\n";

    private static final String tab = "\t";

    public static Boolean generate(File file, String basePackage) {
        try (FileWriter writer = new FileWriter(file)) {
            List<String> methodsList = scan(basePackage);
            for (String method : methodsList) {
                writer.write(method);
                writer.write(ln);
                writer.write(ln);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public static List<String> scan(String basePackage) {
        long start = System.currentTimeMillis();
        final List<String> methodsList = new ArrayList<>();
        try {
            Set<Class<?>> classes;
            if (!StringUtil.isEmpty(basePackage)) {
                classes = new HashSet<>(ClassUtil.getClasses(basePackage));
            } else {
                throw new IllegalArgumentException(basePackage);
            }
            for (Class<?> clazz : classes) {
                if (clazz.isAnnotationPresent(LogGenerated.class)) {
                    methodsList.add(generateLogRecorder(clazz));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        long end = System.currentTimeMillis();
        System.out.printf("LogGenerator:%dms\n", end - start);
        return methodsList;
    }

    private static String generateLogRecorder(Class<?> clazz) {
        System.out.println("Generating:" + clazz.getName());
        LogGenerated generated = clazz.getDeclaredAnnotation(LogGenerated.class);
        Class<?> target = generated.value();

        Actor actorAnnotation = clazz.getAnnotation(Actor.class);
        String actor;
        if (actorAnnotation != null) {
            actor = actorAnnotation.value();
        } else {
            actor = "用户";
        }

        Field[] declaredFields = clazz.getDeclaredFields();
        StringBuilder builder = new StringBuilder();
        builder.append("OperationLog ").append("generateLogWith").append(clazz.getSimpleName()).append("(")
                .append(clazz.getSimpleName()).append(" ").append("info").append(",")
                .append(target.getSimpleName()).append(" ").append("target").append(")")
                .append("{").append(ln);
        builder.append(tab).append("def log=new OperationLog()").append(ln);
        builder.append(tab).append("def originData=target.toString()").append(ln);
        builder.append(tab).append("def builder=new StringBuilder()").append(ln);
        try {
            for (Field field : declaredFields) {
                LogFieldIgnore ignore = field.getAnnotation(LogFieldIgnore.class);
                if (ignore != null) {
                    continue;
                }
                LogFieldDescription logFieldDescription = field.getAnnotation(LogFieldDescription.class);
                String description = null;
                String aimedField;
                if (logFieldDescription == null) {
                    aimedField = field.getName();
                } else {
                    description = logFieldDescription.value();
                    aimedField = logFieldDescription.targetField();
                    if (aimedField == null || aimedField.trim().equals("")) {
                        aimedField = field.getName();
                    }
                }
                Field targetField = target.getDeclaredField(aimedField);
                if (description == null || description.trim().equals("")) {
                    LogFieldDescription annotation = targetField.getAnnotation(LogFieldDescription.class);
                    description = annotation.value();
                }
                if (description == null || description.trim().equals("")) {
                    System.out.println("Error! Cannot find description for: " + field.getName());
                }

                if (field.getType().equals(String.class)) {
                    builder.append(tab).append("if(!StringUtil.isEmpty(info.").append(field.getName()).append(")){").append(ln);
                    builder.append(tab).append(tab).append("builder.append(\"").append(actor).append("\")")
                            .append(".append(\"").append("修改了").append(description).append("为： \")")
                            .append(".append(").append("info.").append(field.getName())
                            .append(").append(ln)").append(ln);
                    builder.append(tab).append(tab).append("target.").append(aimedField).append(" = ").append("info.").append(field.getName()).append(ln);
                    builder.append(tab).append("}").append(ln);
                } else {
                    builder.append(tab).append("if(info.").append(field.getName()).append("!=null){").append(ln);
                    builder.append(tab).append(tab).append("builder.append(\"").append(actor).append("\")")
                            .append(".append(\"").append("修改了").append(description).append("为： \")")
                            .append(".append(").append("info.").append(field.getName())
                            .append(").append(ln)").append(ln);
                    builder.append(tab).append(tab).append("target.").append(aimedField).append(" = ").append("info.").append(field.getName()).append(ln);
                    builder.append(tab).append("}").append(ln);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
        builder.append(tab).append("def newData=info.toString()").append(ln);
        builder.append(tab).append("log.originData=originData").append(ln);
        builder.append(tab).append("log.newData=newData").append(ln);
        builder.append(tab).append("log.description=builder.toString()").append(ln);
        builder.append(tab).append("log").append(ln);
        builder.append("}").append(ln);
        return builder.toString();
    }
}
