package org.brynhild.graduation.common.parser;

import jakarta.annotation.PostConstruct;
import lombok.Data;
import org.brynhild.graduation.common.config.DynamicLoader;
import org.brynhild.graduation.common.tag.GenerateParser;
import org.brynhild.graduation.common.utils.ApplicationUtil;
import org.brynhild.graduation.common.utils.ClassUtil;
import org.brynhild.graduation.common.utils.StringUtil;
import org.springframework.beans.BeansException;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

import java.lang.reflect.*;
import java.util.*;

@Data
@Configuration
@ConfigurationProperties(prefix = "brynhild.common.generator")
@ComponentScan(basePackages = "org.brynhild.graduation.common")
public class ParserClassGenerator implements ApplicationContextAware {

    private static final Map<Class<?>, Class<?>> mappings = new HashMap<>();
    private static final Map<Class<?>, String> defaultValueMap = new HashMap<>();
    private static final Set<Class<?>> basicTypeSet = new HashSet<>();
    private static final Map<String, Class<?>> basicTypeStringMap = new HashMap<>();

    static {
        mappings.put(int.class, Integer.class);
        mappings.put(byte.class, Byte.class);
        mappings.put(char.class, Character.class);
        mappings.put(short.class, Short.class);
        mappings.put(long.class, Long.class);
        mappings.put(boolean.class, Boolean.class);
        mappings.put(float.class, Float.class);
        mappings.put(double.class, Double.class);

        defaultValueMap.put(int.class, "return 0;");
        defaultValueMap.put(byte.class, "return 0;");
        defaultValueMap.put(char.class, "return 0;");
        defaultValueMap.put(short.class, "return 0;");
        defaultValueMap.put(long.class, "return 0;");
        defaultValueMap.put(boolean.class, "return false;");
        defaultValueMap.put(float.class, "return 0;");
        defaultValueMap.put(double.class, "return 0;");

        List<Class<?>> registerClass = Arrays.asList(int.class, Integer.class, byte.class, Byte.class, char.class, Character.class,
                short.class, Short.class, long.class, Long.class, boolean.class, Boolean.class, float.class, Float.class,
                double.class, Double.class, String.class);
        for (Class<?> clazz : registerClass) {
            basicTypeStringMap.put(clazz.getName(), clazz);
        }
        basicTypeSet.addAll(registerClass);
    }

    public final String ln = "\r\n";
    public Map<String, Parser> parserMap = new HashMap<>();
    private String basePackage;
    private int index = 0;


    private ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
        ApplicationUtil.setApplicationContext(applicationContext);
    }

    @PostConstruct
    public void scan() {
        try {
            if (!StringUtil.isEmpty(basePackage)) {
                long start = System.currentTimeMillis();
                Set<Class<?>> classes = ClassUtil.getClasses(basePackage);
                long point1 = System.currentTimeMillis();
                System.out.printf("ScanAnnotation:%dms\n", point1 - start);
                for (Class<?> clazz : classes) {
                    if (clazz.isAnnotationPresent(GenerateParser.class)) {
                        generateParser(clazz);
                    }
                }
                long end = System.currentTimeMillis();
                System.out.printf("ParseGenerator:%dms\n", end - start);
                System.out.printf("Generate Parser Count:%d\n", StaticParser.parserMap.size());
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void generateParser(Class<?> clazz) {
        if (StaticParser.getParser(clazz) != null) {
            return;
        }
        Field[] declaredFields = clazz.getDeclaredFields();
        Method[] methods = clazz.getMethods();
        StringBuilder builder = new StringBuilder();
        builder.append(ParserClassGenerator.class.getPackage()).append(";").append(ln);
        builder.append("import java.util.Map;").append(ln);
        builder.append("import java.util.Set;").append(ln);
        builder.append("import java.util.List;").append(ln);
        builder.append("import java.util.ArrayList;").append(ln);
        builder.append("import org.brynhild.common.parser.Parser;").append(ln);
        builder.append("import org.brynhild.common.parser.StaticParser;").append(ln);
        builder.append("import org.brynhild.common.utils.TypeCastUtil;").append(ln);
        String javaName = getClassName();
        builder.append("public class ").append(javaName).append(" implements Parser{").append(ln);
        builder.append("public ").append(clazz.getName()).append(" parse(Map<String,Object> values) {").append(ln);
        builder.append("if(values==null)return null;").append(ln);
        builder.append(clazz.getName()).append(" ret=new ").append(clazz.getName()).append("();").append(ln);
        Map<String, Method> methodMap = new HashMap<>();
        int arrayIndex = 0;
        int tmpIndex = 0;
        for (Method method : methods) {
            String methodName = method.getName();
            String substring = null;
            if (methodName.startsWith("set")) {
                substring = methodName.substring(3);

            } else if (methodName.startsWith("is")) {
                substring = methodName.substring(2);
            } else {
                continue;
            }
            char c = substring.charAt(0);
            if (Character.isUpperCase(c)) {
                c = Character.toLowerCase(c);
            }
            String pair = c + substring.substring(1);
            methodMap.put(pair, method);
        }
        for (Field field : declaredFields) {
            field.setAccessible(true);
            Class<?> type = field.getType();
            Method method = methodMap.get(field.getName());
            builder.append("if(values.get(\"").append(field.getName()).append("\")!=null){").append(ln);
            if (isBasicType(type)) {
//                if (isOriginBasicType(type)) {
//                    throw new RuntimeException("Cannot create parser because primitive type is unsupported!\n" +
//                            "Class:" + clazz.getName() +
//                            "Field:" + field.getName());
//                }
                Class<?>[] parameterTypes = method.getParameterTypes();
                String parameterType = parameterTypes[0].getName();

                builder.append("ret.").append(method.getName()).append("(");
                builder.append("org.brynhild.common.parser.convertor.TypeConvertorMap.convert(").append(parameterType).append(".class").append(",values.get(\"").append(field.getName()).append("\").getClass()").append(",values.get(\"").append(field.getName()).append("\"))");
                builder.append(");").append(ln);

//                BasicTypeConvertor convertor = convertorMap.getConvertor(typeClass);
//                String convert = convertor.convert();


            } else if (isArrayType(type)) {
                throw new RuntimeException("Cannot create parser because array is unsupported!");
//                Class<?> componentType = type.getComponentType();
//                String arrayName = "array" + arrayIndex++ + "";
//                String tmpName = "tmp" + tmpIndex++ + "";
//                builder.append(componentType.getName()).append("[] ").append(arrayName).append("= null;").append(ln);
//                builder.append("List<Object> ").append(tmpName).append("= null;").append(ln);
//                builder.append(tmpName).append("= TypeCastUtil.castToList(");
//                builder.append("values.get(\"").append(field.getName()).append("\")");
//                builder.append(");").append(ln);
//                String size = "size" + sizeIndex++ + "";
//                builder.append("int ").append(size).append("=").append(tmpName).append(".size();").append(ln);
//                builder.append(arrayName).append("=new ").append(componentType.getName()).append("[").append(size).append("];").append(ln);
//                builder.append("for(int i=0;i<").append(size).append(";i++) {").append(ln);
//                if (isBasicType(componentType)) {
//                    builder.append(arrayName).append("[i]=").append(tmpName).append(".get(i);").append(ln);
//                } else {
//                    builder.append(arrayName).append("[i]=").append("(").append(componentType.getName()).append(")com.brynhild.common.paser.StaticParser.parserMap.get(\"");
//                    builder.append(componentType.getName()).append("\").parse(");
//                    builder.append("TypeCastUtil.castToMap(");
//                    builder.append(tmpName).append(".get(i),String.class,Object.class));");
//                    builder.append(ln);
//                }
//                builder.append("}").append(ln);

            } else if (isListType(type)) {
                ParameterizedType pt = (ParameterizedType) field.getGenericType();
                Type actualTypeArgument = pt.getActualTypeArguments()[0];
                try {
                    Class<?> componentType = ClassLoader.getSystemClassLoader().loadClass(actualTypeArgument.getTypeName());
                    String arrayName = "array" + arrayIndex++ + "";
                    String tmpName = "tmp" + tmpIndex++ + "";
                    builder.append("List<").append(componentType.getName()).append("> ").append(arrayName).append("= null;").append(ln);
                    if (isBasicType(componentType)) {
                        builder.append(arrayName).append("= TypeCastUtil.castToList(");
                        builder.append("values.get(\"").append(field.getName()).append("\")");
                        builder.append(",").append(componentType.getName()).append(".class);").append(ln);
                    } else {
                        builder.append(arrayName).append("= new ArrayList<>();").append(ln);
                        builder.append("List<Object> ").append(tmpName).append("= TypeCastUtil.castToList(");
                        builder.append("values.get(\"").append(field.getName()).append("\")");
                        builder.append(",").append(Object.class.getName()).append(".class);").append(ln);
                        Parser parser = parserMap.get(componentType.getName());
                        if (parser == null) {
                            generateParser(componentType);
                        }
                        builder.append("for(Object obj:").append(tmpName).append("){").append(ln);
                        builder.append(arrayName).append(".add((").append(componentType.getName()).append(")org.brynhild.common.parser.StaticParser.parserMap.get(\"");
                        builder.append(componentType.getName()).append("\").parse(TypeCastUtil.castToMap(obj,String.class,Object.class)));").append(ln);
                        builder.append("}").append(ln);

                    }
                    builder.append("ret.").append(method.getName()).append("(").append(arrayName).append(");").append(ln);
                } catch (Exception e) {
                    e.printStackTrace();
                }

            } else {
                Parser parser = parserMap.get(type.getName());
                if (parser == null) {
                    generateParser(type);
                }
                builder.append("ret.").append(method.getName()).append("(");
                builder.append("(").append(field.getType().getName()).append(")");
                builder.append("org.brynhild.common.parser.StaticParser.parserMap.get(\"");
                builder.append(field.getType().getName()).append("\").parse(");
                builder.append("TypeCastUtil.castToMap(");
                builder.append("values.get(\"").append(field.getName()).append("\")");
                builder.append(",String.class,Object.class)");
                builder.append("));").append(ln);
            }
            builder.append("}").append(ln);
        }
        builder.append("return ret;").append(ln);
        builder.append("}").append(ln);
        builder.append("}").append(ln);
        try {
            Map<String, byte[]> bytecode = DynamicLoader.compile(javaName + ".java", builder.toString());
            DynamicLoader.MemoryClassLoader classLoader = new DynamicLoader.MemoryClassLoader(bytecode);
            Class<?> generateClass = classLoader.loadClass(ParserClassGenerator.class.getPackage().getName() + "." + javaName);
            System.out.println("register...." + javaName);
            ApplicationUtil.register(applicationContext, javaName, builder.toString());
            System.out.println("register...." + javaName + "  FINISHED");
            Constructor<?> constructor = generateClass.getConstructor();
            Object instance = constructor.newInstance();
            parserMap.put(clazz.getName(), (Parser) instance);
            StaticParser.parserMap.put(clazz.getName(), (Parser) instance);
            classLoader.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private boolean isBasicType(Class<?> type) {
        return basicTypeSet.contains(type);
    }

    private boolean isArrayType(Class<?> type) {
        return type.isArray();
    }

    private boolean isListType(Class<?> type) {
        try {
            return List.class.isAssignableFrom(type) || type.newInstance() instanceof List;
        } catch (Exception e) {
            return false;
        }
    }

    private boolean isOriginBasicType(Class<?> type) {
        return mappings.containsKey(type);
    }

    private String getClassName() {
        return "$Parser" + index++ + "";
    }

}
