package org.brynhild.graduation.common.transfer.util;

import lombok.AllArgsConstructor;
import lombok.Data;

import java.util.ArrayList;
import java.util.List;

public class HttpURLBuilder {

    private final String baseUrl;

    private final List<KeyValue> params = new ArrayList<>();

    public HttpURLBuilder(String baseUrl) {
        this.baseUrl = baseUrl;
    }

    public HttpURLBuilder addParam(String key, String value) {
        this.params.add(new KeyValue(key, value));
        return this;
    }

    public String build() {
        StringBuilder builder = new StringBuilder();
        builder.append(baseUrl);
        if (params.size() == 0) {
            return builder.toString();
        }
        KeyValue keyValue = params.get(0);
        builder.append("?").append(keyValue.getKey()).append("=").append(keyValue.getValue());
        for (int i = 1; i < params.size(); i++) {
            keyValue = params.get(i);
            builder.append("&").append(keyValue.getKey()).append("=").append(keyValue.getValue());
        }
        return builder.toString();
    }

    @Data
    @AllArgsConstructor
    static class KeyValue {
        private String key;
        private String value;
    }
}
