package org.brynhild.graduation.common.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

public class TimeUtil {

    private static final SimpleDateFormat timeStringFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static final SimpleDateFormat timeNumberStringFormat = new SimpleDateFormat("yyyyMMddHHmmss");
    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
    private static TimeZone currentTimeZone;

    static {
        currentTimeZone = TimeZone.getTimeZone("GMT+8");
    }

    public static void setTimeZone(TimeZone timeZone) {
        TimeZone.setDefault(timeZone);
    }

    public static String getCurrentTimeString() {
        return timeStringFormat.format(new Date());
    }

    public static String getCurrentTimeNumberString() {
        return timeNumberStringFormat.format(new Date());
    }

    public static String convertToTimeString(String timeNumberString) {
        try {
            return timeStringFormat.format(timeNumberStringFormat.parse(timeNumberString));
        } catch (Exception e) {
            return null;
        }
    }

    public static String convertToTimeNumberString(String timeString) {
        try {
            return timeNumberStringFormat.format(timeStringFormat.parse(timeString));
        } catch (Exception e) {
            return null;
        }
    }

    public static Date getMonthDate(Date startDate, int month) {
        LocalDateTime localDateTime = startDate.toInstant()
                .atZone(ZoneId.systemDefault())
                .toLocalDateTime().plusMonths(month);
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    public static Date getMonthDate(int month) {
        return getMonthDate(new Date(), month);
    }

    public static String getMonthDateString(int month) {
        return timeStringFormat.format(getMonthDate(month));
    }

    public static Long getTimeDifference(String time) {
        try {
            Date now = new Date();
            Date originDate = timeStringFormat.parse(time);
            return now.getTime() - originDate.getTime();
        } catch (Exception e) {
            e.printStackTrace();
            return 0L;
        }
    }

    public static String renewValidity(String origin, int month) {
        try {
            Date now = new Date();
            Date originDate = timeStringFormat.parse(origin);
            Date newDate = null;
            if (originDate.getTime() <= now.getTime()) {
                newDate = getMonthDate(now, month);
            } else {
                newDate = getMonthDate(originDate, month);
            }
            return timeStringFormat.format(newDate);
        } catch (Exception e) {
            e.printStackTrace();
            return origin;
        }
    }

    public static boolean expire(String time) {
        try {
            Date now = new Date();
            Date origin = timeStringFormat.parse(time);
            return origin.getTime() < now.getTime();
        } catch (Exception e) {
            e.printStackTrace();
            return true;
        }
    }

    public static List<String> generateDateSection(Date begin, Date end, SimpleDateFormat format, Integer step) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(begin);
        List<String> dateList = new ArrayList<>();
        while (begin.before(end)) {
            dateList.add(format.format(begin));
            calendar.add(Calendar.DATE, step);
            begin = calendar.getTime();
        }
        return dateList;
    }

    public static List<String> generateDateSection(Date begin, Date end, Integer step) {
        return generateDateSection(begin, end, DATE_FORMAT, step);
    }

    public static List<String> generateDateSection(Date begin, Integer duration, Integer step) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(begin);
        calendar.add(Calendar.DATE, duration);
        return generateDateSection(begin, calendar.getTime(), step);
    }


    public static List<String> generateDateSection(Date begin, Date end, SimpleDateFormat format) {
        return generateDateSection(begin, end, format, 1);
    }

    public static List<String> generateDateSection(Date begin, Integer duration, SimpleDateFormat format) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(begin);
        calendar.add(Calendar.DATE, duration);
        return generateDateSection(begin, calendar.getTime(), format);
    }

    public static List<String> generateDateSection(Date begin, Date end) {
        return generateDateSection(begin, end, DATE_FORMAT);
    }

    public static List<String> generateDateSection(Date begin, Integer duration) {
        return generateDateSection(begin, duration, DATE_FORMAT);
    }

    public static Date parseDate(String date) throws ParseException {
        return DATE_FORMAT.parse(date);
    }

    public static Date parseDate(String date, SimpleDateFormat format) throws ParseException {
        return format.parse(date);
    }
}
