package org.brynhild.graduation.device.factory

import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.persistence.device.entity.Area
import org.brynhild.graduation.persistence.device.entity.Credential
import org.brynhild.graduation.persistence.device.entity.Device
import org.brynhild.graduation.persistence.device.entity.DeviceRelationship
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.transfer.device.vo.*
import org.springframework.stereotype.Component

@Component
class OperationLogFactory {
    private final String ln = "\r\n";


    OperationLog generateLogWithAdminAddArea(AdminAddArea info, Area target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (info.owner != null) {
            builder.append("管理员").append("修改了负责人为： ").append(info.owner).append(ln)
            target.owner = info.owner
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("管理员").append("修改了区域为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.note)) {
            builder.append("管理员").append("修改了备注信息为： ").append(info.note).append(ln)
            target.note = info.note
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminAddCredential(AdminAddCredential info, Credential target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (info.userId != null) {
            builder.append("管理员").append("修改了用户id为： ").append(info.userId).append(ln)
            target.userId = info.userId
        }
        if (!StringUtil.isEmpty(info.userCredential)) {
            builder.append("管理员").append("修改了用户身份标识符为： ").append(info.userCredential).append(ln)
            target.userCredential = info.userCredential
        }
        if (info.areaId != null) {
            builder.append("管理员").append("修改了区域id为： ").append(info.areaId).append(ln)
            target.areaId = info.areaId
        }
        if (info.deviceId != null) {
            builder.append("管理员").append("修改了设备id为： ").append(info.deviceId).append(ln)
            target.deviceId = info.deviceId
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminModifyDevice(AdminModifyDevice info, Device target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (info.owner != null) {
            builder.append("管理员").append("修改了所有者为： ").append(info.owner).append(ln)
            target.owner = info.owner
        }
        if (info.area != null) {
            builder.append("管理员").append("修改了区域为： ").append(info.area).append(ln)
            target.area = info.area
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("管理员").append("修改了设备名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (info.active != null) {
            builder.append("管理员").append("修改了激活状态为： ").append(info.active).append(ln)
            target.active = info.active
        }
        if (!StringUtil.isEmpty(info.note)) {
            builder.append("管理员").append("修改了备注信息为： ").append(info.note).append(ln)
            target.note = info.note
        }
        if (!StringUtil.isEmpty(info.apSsid)) {
            builder.append("管理员").append("修改了AP SSID为： ").append(info.apSsid).append(ln)
            target.apSsid = info.apSsid
        }
        if (!StringUtil.isEmpty(info.apPassword)) {
            builder.append("管理员").append("修改了AP PASSWORD为： ").append(info.apPassword).append(ln)
            target.apPassword = info.apPassword
        }
        if (!StringUtil.isEmpty(info.staSsid)) {
            builder.append("管理员").append("修改了STA SSID为： ").append(info.staSsid).append(ln)
            target.staSsid = info.staSsid
        }
        if (!StringUtil.isEmpty(info.staPassword)) {
            builder.append("管理员").append("修改了STA PASSWORD为： ").append(info.staPassword).append(ln)
            target.staPassword = info.staPassword
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminAddRelationship(AdminAddRelationship info, DeviceRelationship target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (info.master != null) {
            builder.append("用户").append("修改了主设备为： ").append(info.master).append(ln)
            target.master = info.master
        }
        if (info.slave != null) {
            builder.append("用户").append("修改了从设备为： ").append(info.slave).append(ln)
            target.slave = info.slave
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithBindDevice(BindDevice info, Device target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.mac)) {
            builder.append("管理员").append("修改了mac地址为： ").append(info.mac).append(ln)
            target.mac = info.mac
        }
        if (info.type != null) {
            builder.append("管理员").append("修改了设备类型为： ").append(info.type).append(ln)
            target.type = info.type
        }
        if (info.area != null) {
            builder.append("管理员").append("修改了区域为： ").append(info.area).append(ln)
            target.area = info.area
        }
        if (!StringUtil.isEmpty(info.note)) {
            builder.append("管理员").append("修改了备注信息为： ").append(info.note).append(ln)
            target.note = info.note
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("管理员").append("修改了设备名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.apSsid)) {
            builder.append("管理员").append("修改了AP SSID为： ").append(info.apSsid).append(ln)
            target.apSsid = info.apSsid
        }
        if (!StringUtil.isEmpty(info.apPassword)) {
            builder.append("管理员").append("修改了AP PASSWORD为： ").append(info.apPassword).append(ln)
            target.apPassword = info.apPassword
        }
        if (!StringUtil.isEmpty(info.staSsid)) {
            builder.append("管理员").append("修改了STA SSID为： ").append(info.staSsid).append(ln)
            target.staSsid = info.staSsid
        }
        if (!StringUtil.isEmpty(info.staPassword)) {
            builder.append("管理员").append("修改了STA PASSWORD为： ").append(info.staPassword).append(ln)
            target.staPassword = info.staPassword
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminModifyArea(AdminModifyArea info, Area target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (info.owner != null) {
            builder.append("管理员").append("修改了负责人为： ").append(info.owner).append(ln)
            target.owner = info.owner
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("管理员").append("修改了区域为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.note)) {
            builder.append("管理员").append("修改了备注信息为： ").append(info.note).append(ln)
            target.note = info.note
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }
}
