package org.brynhild.graduation.device.util;


import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.MqttConstant;
import org.brynhild.graduation.common.utils.RandomCodeUtil;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Component
@RequiredArgsConstructor
public class RedisUtil {

    private final RedisTemplate<String, Object> redisTemplate;

    public String getVerifyCode(String prefix, Object value) {
        Object key = null;
        String randomCode = null;
        do {
            randomCode = RandomCodeUtil.getRandomNumberCode(6);
            key = redisTemplate.opsForValue().get(prefix + randomCode);
        } while (key != null);
        redisTemplate.opsForValue().set(prefix + randomCode, value, 5, TimeUnit.MINUTES);
        return randomCode;
    }

    public Object getVerifyCodeData(String prefix, String key) {
        return redisTemplate.opsForValue().get(prefix + key);
    }

    public void setDeviceReportData(String macAddress, String data) {
        redisTemplate.opsForValue().set(MqttConstant.DEVICE_DATA_PREFIX + macAddress, data, 5, TimeUnit.MINUTES);
    }

    public String getDeviceReportData(String macAddress) {
        final Object o = redisTemplate.opsForValue().get(MqttConstant.DEVICE_DATA_PREFIX + macAddress);
        if (o == null) {
            return null;
        }
        return (String) o;
    }

    public void refreshDeviceHeartBeat(String macAddress) {
        redisTemplate.opsForValue().set(MqttConstant.DEVICE_HEARTBEAT_PREFIX + macAddress, "online", 30, TimeUnit.SECONDS);
    }

    public boolean isDeviceOnline(String macAddress) {
        return redisTemplate.opsForValue().get(MqttConstant.DEVICE_HEARTBEAT_PREFIX + macAddress) != null;
    }

    public boolean shouldNotify(String macAddress, String contentJson) {
        final Object record = redisTemplate.opsForValue().get(MqttConstant.DEVICE_NOTIFY_RECORD + macAddress);
        if (record == null) {
            redisTemplate.opsForValue().set(MqttConstant.DEVICE_NOTIFY_RECORD + macAddress, contentJson, 5, TimeUnit.MINUTES);
            return true;
        }
        return false;
    }
}
