package org.brynhild.graduation.gateway.listener

import jakarta.annotation.Resource
import lombok.RequiredArgsConstructor
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.gateway.rule.TokenValidRule
import org.brynhild.graduation.transfer.event.bo.GatewayDeleteToken
import org.brynhild.graduation.transfer.event.bo.GatewayUpdateToken
import org.springframework.amqp.rabbit.annotation.RabbitHandler
import org.springframework.amqp.rabbit.annotation.RabbitListener
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
@RabbitListener(queues = RabbitQueueConstant.USER_TOKEN)
@RequiredArgsConstructor
class UserTokenListener {
    private TokenValidRule rule

    @RabbitHandler
    void handle(byte[] bytes) {
        try {
            Object objectFromBytes = ByteTransferUtil.getObjectFromBytes(bytes)
            switch (objectFromBytes) {
                case GatewayUpdateToken:
                    rule.notifyUserTokenChange(objectFromBytes.username, objectFromBytes.token)
                    break
                case GatewayDeleteToken:
                    rule.deleteUserToken(objectFromBytes.username)
                    break
                default:
                    throw new IllegalStateException("Unexpected value: " + objectFromBytes)
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }

    @Autowired
    @Resource
    void setRule(TokenValidRule rule) {
        this.rule = rule
    }
}
