package org.brynhild.graduation.persistence.common.repository;

import jakarta.transaction.Transactional;
import org.brynhild.graduation.common.utils.TimeUtil;
import org.brynhild.graduation.persistence.common.entity.LogicDeleteEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.NoRepositoryBean;

@NoRepositoryBean
public interface LogicDeleteRepository<T extends LogicDeleteEntity> extends JpaRepository<T, Long>, JpaSpecificationExecutor<T> {


    @Modifying
    @Transactional(rollbackOn = Exception.class)
    @Query("UPDATE #{#entityName} t SET t.deleteAt=?2,t.updateAt=?3 WHERE t.id=?1")
    void logicDelete(Long id, String update, String delete);

    default void logicDelete(Long id) {
        String now = TimeUtil.getCurrentTimeString();
        logicDelete(id, now, now);
    }


    @Modifying
    @Transactional
    @Query("UPDATE #{#entityName} t SET t.deleteAt=NULL,t.updateAt=?2 WHERE t.id=?1")
    void undoDelete(Long id, String now);

    default void undoDelete(Long id) {
        undoDelete(id, TimeUtil.getCurrentTimeString());
    }
}
