package org.brynhild.graduation.persistence.device.entity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.Table;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.brynhild.graduation.common.utils.ByteTransferUtil;
import org.brynhild.graduation.persistence.common.constant.CommonDBStringDefinition;
import org.brynhild.graduation.persistence.common.entity.LogicDeleteEntity;
import org.hibernate.annotations.Where;

import java.util.Arrays;

@EqualsAndHashCode(callSuper = true)
@Entity
@Table(name = "finger_print_data")
@Data
@Where(clause = CommonDBStringDefinition.LOGIC_DELETE_WHERE_CLAUSE)
@NoArgsConstructor
public class FingerPrintData extends LogicDeleteEntity {

    @Column(name = "mac")
    private String macAddress;
    private Integer uid;
    private byte[] data;

    public FingerPrintData(byte[] buffer) {
        if (buffer == null || buffer.length != 536) {
            throw new IllegalArgumentException("buffer array is NULL or length mismatch!");
        }
        byte[] idBuffer = new byte[4];
        byte[] dataBuffer = new byte[4 * 128];
        macAddress = new String(buffer, 0, 17);
        System.arraycopy(buffer, 20, idBuffer, 0, 4);
        System.arraycopy(buffer, 24, dataBuffer, 0, 4 * 128);
        uid = ByteTransferUtil.byte2Int(idBuffer, true);
        data = dataBuffer;
    }

    public byte[] getData() {
        return data;
    }

    public void setData(byte[] data) {
        this.data = data;
    }

    public byte[] convertToBytes() {
        byte[] buffer = new byte[536];
        System.arraycopy(macAddress.getBytes(), 0, buffer, 0, 17);
        System.arraycopy(ByteTransferUtil.int2Byte(uid, true), 0, buffer, 20, 4);
        System.arraycopy(data, 0, buffer, 24, 4 * 128);
        return buffer;
    }

    @Override
    public String toString() {
        return "FingerPrintData{" +
                "macAddress='" + macAddress + '\'' +
                ", id=" + uid +
                ", data=" + Arrays.toString(data) +
                '}';
    }
}
