package org.brynhild.graduation.persistence.user.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.Table;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.brynhild.graduation.common.constant.UserStateConstant;
import org.brynhild.graduation.common.tag.LogFieldDescription;
import org.brynhild.graduation.persistence.common.constant.CommonDBStringDefinition;
import org.brynhild.graduation.persistence.common.entity.LogicDeleteEntity;
import org.hibernate.annotations.Where;

import java.io.Serializable;

@EqualsAndHashCode(callSuper = true)
@Entity
@Table(name = "user")
@Data
@Where(clause = CommonDBStringDefinition.LOGIC_DELETE_WHERE_CLAUSE)
@NoArgsConstructor
public class User extends LogicDeleteEntity implements Serializable {
    @LogFieldDescription("用户名")
    private String username;
    @JsonIgnore
    @LogFieldDescription("密码")
    private String password;
    @LogFieldDescription("电子邮件地址")
    private String email;
    @LogFieldDescription("用户状态")
    private Integer state = UserStateConstant.NOT_ACTIVE;
    @Column(name = "last_login")
    private String lastLogin = AccountConstant.DEFAULT_LOGIN_TIME;
    @LogFieldDescription("头像链接")
    private String avatar = AccountConstant.DEFAULT_AVATAR;
    @LogFieldDescription("用户类型")
    private Integer type = AccountConstant.USER;
    @LogFieldDescription("姓名")
    private String name;
    @LogFieldDescription("性别")
    private String sex;
    @Column(name = "department_id")
    @LogFieldDescription("部门编号")
    private Long departmentId;
    @LogFieldDescription("联系电话")
    private String tel;
    @LogFieldDescription("学号/工号")
    private String no;
}
