package org.brynhild.graduation.builder.event

import jakarta.annotation.Resource
import org.brynhild.graduation.builder.executor.BuildTaskExecutor
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.transfer.trigger.BuildTask
import org.springframework.amqp.rabbit.annotation.RabbitHandler
import org.springframework.amqp.rabbit.annotation.RabbitListener
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
@RabbitListener(queues = RabbitQueueConstant.BUILD_EVENT)
class BuildEventListener {
    private BuildTaskExecutor buildTaskExecutor

    @RabbitHandler
    void execute(byte[] bytes) {
        try {
            Object obj = ByteTransferUtil.getObjectFromBytes(bytes)
            switch (obj) {
                case BuildTask:
                    buildTaskExecutor.execute(obj)
                    break
                default: throw new IllegalStateException("Unexpected value: " + obj)
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }

    @Autowired
    @Resource
    void setBuildTaskExecutor(BuildTaskExecutor buildTaskExecutor) {
        this.buildTaskExecutor = buildTaskExecutor
    }
}
