package org.brynhild.graduation.common.parser;

import org.brynhild.graduation.common.tag.DataClassIgnore;
import org.brynhild.graduation.common.utils.ClassUtil;
import org.brynhild.graduation.common.utils.FileUtil;
import org.brynhild.graduation.common.utils.StringUtil;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;

public class DataClassGenerator {
    private static final String ln = "\r\n";

    private static final String tab = "\t";

    public static Boolean generate(String dir, String basePackage, String targetPackage) {

        FileUtil.deleteFile(new File(dir));

        Map<String, String> clazzMap = scan(basePackage, targetPackage);
        try {
            for (Map.Entry<String, String> entry : clazzMap.entrySet()) {
                String remain = entry.getKey().substring(basePackage.length() + 1);
                String targetDirPath = "";
                if (entry.getKey().lastIndexOf(".") > basePackage.length() + 1) {
                    targetDirPath = entry.getKey().substring(basePackage.length() + 1, entry.getKey().lastIndexOf("."));
                }


                File file = new File(dir + File.separator + remain.replaceAll("\\.", Matcher.quoteReplacement(File.separator)) + ".kt");
                File targetDir = new File(dir + File.separator + targetDirPath);
                if (!targetDir.exists()) {
                    targetDir.mkdirs();
                }
                try (FileWriter writer = new FileWriter(file)) {
                    writer.write(entry.getValue());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    private static Map<String, String> scan(String basePackage, String targetPackage) {
        long start = System.currentTimeMillis();
        final Map<String, String> clazzMap = new HashMap<>();
        try {
            Set<Class<?>> classes;
            if (!StringUtil.isEmpty(basePackage)) {
                classes = new HashSet<>(ClassUtil.getClasses(basePackage));
            } else {
                throw new IllegalArgumentException(basePackage);
            }
            for (Class<?> clazz : classes) {
                if (!clazz.isAnnotationPresent(DataClassIgnore.class)) {
                    if (clazzMap.containsKey(clazz.getSimpleName())) {
                        System.out.println("[ERROR] Duplicate key founded when generate data class: " + clazz.getName() + " ,skip!");
                        continue;
                    }
                    String generateDataClass = generateDataClass(clazz, targetPackage);
                    clazzMap.put(clazz.getName(), generateDataClass);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        long end = System.currentTimeMillis();
        System.out.printf("DataClassGenerator:%dms\n", end - start);
        return clazzMap;
    }

    private static String generateDataClass(Class<?> clazz, String targetPackage) {
        System.out.println("Generating data class:" + clazz.getName());

        Field[] declaredFields = clazz.getDeclaredFields();
        StringBuilder builder = new StringBuilder();
        builder.append("package ").append(targetPackage).append(ln).append(ln);

        builder.append("data class ").append(clazz.getSimpleName()).append("(").append(ln);

        try {
            for (Field field : declaredFields) {
                boolean allowNull = true;
                NotBlank notBlank = field.getAnnotation(NotBlank.class);
                if (notBlank != null) {
                    allowNull = false;
                }
                NotNull notNull = field.getAnnotation(NotNull.class);
                if (notNull != null) {
                    allowNull = false;
                }

                builder.append(tab).append("val ").append(field.getName()).append(":");
                if (field.getType().equals(Integer.class)) {
                    builder.append("Int");
                } else {
                    builder.append(field.getType().getSimpleName());
                }
                if (allowNull) {
                    builder.append("?");
                }
                builder.append(",").append(ln);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
        builder.append(")").append(ln);
        return builder.toString();
    }
}
