package org.brynhild.graduation.common.parser;

import org.brynhild.graduation.common.tag.SpecificationFieldMapping;
import org.brynhild.graduation.common.tag.SpecificationGenerated;
import org.brynhild.graduation.common.tag.SpecificationIgnore;
import org.brynhild.graduation.common.utils.ClassUtil;
import org.brynhild.graduation.common.utils.StringUtil;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class SpecificationGenerator {
    private static final String ln = "\r\n";

    private static final String tab = "\t";

    public static Boolean generate(File file, String scanPackage) {
        try (FileWriter writer = new FileWriter(file)) {
            List<String> methodsList = scan(scanPackage);
            for (String method : methodsList) {
                writer.write(method);
                writer.write(ln);
                writer.write(ln);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public static List<String> scan(String basePackage) {
        long start = System.currentTimeMillis();
        final List<String> methodsList = new ArrayList<>();
        final List<Class<?>> addedClasses = new ArrayList<>();
        try {
            Set<Class<?>> classes;
            if (!StringUtil.isEmpty(basePackage)) {
                classes = new HashSet<>(ClassUtil.getClasses(basePackage));
            } else {
                throw new IllegalArgumentException(basePackage);
            }
            for (Class<?> clazz : classes) {
                if (clazz.isAnnotationPresent(SpecificationGenerated.class)) {
                    addedClasses.add(clazz);
                    methodsList.add(generateSpecificationGenerator(clazz));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        long end = System.currentTimeMillis();
        System.out.printf("SpecificationGenerator:%dms\n", end - start);

        List<String> finalContent = new ArrayList<>();
        StringBuilder builder = new StringBuilder();
        builder.append("import jakarta.persistence.criteria.Predicate;").append(ln);
        builder.append("import org.springframework.data.jpa.domain.Specification;").append(ln);
        builder.append("import org.springframework.stereotype.Component;").append(ln);
        builder.append("import org.brynhild.graduation.common.utils.StringUtil;").append(ln);
        for (Class<?> clazz : addedClasses) {
            SpecificationGenerated annotation = clazz.getAnnotation(SpecificationGenerated.class);
            if (annotation == null) {
                throw new IllegalArgumentException("cannot find mapping class");
            }
            Class<?> target = annotation.value();
            if (target == null) {
                throw new IllegalArgumentException("cannot find mapping class");
            }
            builder.append("import ").append(target.getName()).append(";").append(ln);
            builder.append("import ").append(clazz.getName()).append(";").append(ln);
        }

        builder.append(ln);
        builder.append("import java.util.ArrayList;").append(ln);
        builder.append("import java.util.List;").append(ln);
        builder.append(ln);
        builder.append("@Component").append(ln);
        builder.append("public class SpecificationFactory {").append(ln);

        finalContent.add(builder.toString());
        finalContent.addAll(methodsList);
        finalContent.add("}" + ln);

        return finalContent;
    }

    private static String generateSpecificationGenerator(Class<?> clazz) {
        System.out.println("Generating:" + clazz.getName());
        SpecificationGenerated annotation = clazz.getAnnotation(SpecificationGenerated.class);
        if (annotation == null) {
            return "";
        }
        Class<?> target = annotation.value();
        if (target == null) {
            return "";
        }
        Field[] declaredFields = clazz.getDeclaredFields();
        StringBuilder builder = new StringBuilder();
//        builder.append(clazz.getName()).append(ln);
        builder.append("public Specification<").append(target.getSimpleName()).append("> get")
                .append(target.getSimpleName()).append("Specification(").append(clazz.getSimpleName())
                .append(" info){").append(ln);
        builder.append(tab).append("return (root, query, criteriaBuilder) -> {").append(ln);
        builder.append(tab).append(tab).append("List<Predicate> predicateList=new ArrayList<>();").append(ln);
        for (Field field : declaredFields) {
            SpecificationIgnore ignore = field.getAnnotation(SpecificationIgnore.class);
            if (ignore != null) {
                continue;
            }
            SpecificationFieldMapping mapping = field.getAnnotation(SpecificationFieldMapping.class);
            String aimedField;
            if (mapping == null) {
                aimedField = field.getName();
            } else {
                aimedField = mapping.value();
                if (aimedField == null || aimedField.trim().equals("")) {
                    aimedField = field.getName();
                }
            }
            String getterName;
            try {
                getterName = getMethodName(field.getName());
            } catch (Exception e) {
                e.printStackTrace();
                return "";
            }
            if (field.getType().equals(String.class)) {
                builder.append(tab).append(tab).append("if(!StringUtil.isEmpty(info.get").append(getterName).append("())){").append(ln);
                builder.append(tab).append(tab).append(tab).append("predicateList.add(criteriaBuilder.like(root.get(\"")
                        .append(aimedField).append("\").as(String.class),\"%\"+").append("info.get").append(getterName).append("()+\"%\"));").append(ln);
                builder.append(tab).append(tab).append("}").append(ln);
            } else {
                builder.append(tab).append(tab).append("if(info.get").append(getterName).append("()!=null){").append(ln);
                builder.append(tab).append(tab).append(tab).append("predicateList.add(criteriaBuilder.equal(root.get(\"")
                        .append(aimedField).append("\").as(").append(field.getType().getSimpleName()).append(".class),").append("info.get").append(getterName).append("()));").append(ln);
                builder.append(tab).append(tab).append("}").append(ln);
            }
        }
        builder.append(tab).append(tab).append("if(predicateList.isEmpty()){").append(ln);
        builder.append(tab).append(tab).append(tab).append("return null;").append(ln);
        builder.append(tab).append(tab).append("}").append(ln);

        builder.append(tab).append(tab).append("Predicate predicate=predicateList.get(0);").append(ln);
        builder.append(tab).append(tab).append("for(int i=1;i<predicateList.size();i++){").append(ln);
        builder.append(tab).append(tab).append(tab).append("predicate=criteriaBuilder.and(predicate,predicateList.get(i));").append(ln);
        builder.append(tab).append(tab).append("}").append(ln);
        builder.append(tab).append(tab).append("return predicate;").append(ln);
        builder.append(tab).append("};").append(ln);
        builder.append("}").append(ln);
        return builder.toString();
    }

    public static String getMethodName(String fieldName) throws Exception {
        char[] chars = fieldName.toCharArray();
        chars[0] = toUpperCase(chars[0]);
        return String.valueOf(chars);
    }


    public static char toUpperCase(char c) {
        if (97 <= c && c <= 122) {
            c ^= 32;
        }
        return c;
    }
}
