package org.brynhild.graduation.common.parser.convertor;

import java.util.HashMap;
import java.util.Map;

public class TypeConvertorMap {

    private static final Map<Class<?>, BasicTypeConvertor> convertorMap = new HashMap<>();

    static {
        convertorMap.put(Boolean.class, new BooleanConvertor());
        convertorMap.put(boolean.class, new BooleanPrimitiveConvertor());

        convertorMap.put(Byte.class, new ByteConvertor());
        convertorMap.put(byte.class, new BytePrimitiveConvertor());

        convertorMap.put(Double.class, new DoubleConvertor());
        convertorMap.put(double.class, new DoublePrimitiveConvertor());

        convertorMap.put(Float.class, new FloatConvertor());
        convertorMap.put(float.class, new FloatPrimitiveConvertor());

        convertorMap.put(Integer.class, new IntegerConvertor());
        convertorMap.put(int.class, new IntegerConvertor());

        convertorMap.put(Long.class, new LongConvertor());
        convertorMap.put(long.class, new LongPrimitiveConvertor());

        convertorMap.put(Short.class, new ShortConvertor());
        convertorMap.put(short.class, new ShortPrimitiveConvertor());

        convertorMap.put(String.class, new StringConvertor());
    }

    public static BasicTypeConvertor getConvertor(Class<?> type) {
        BasicTypeConvertor convertor = convertorMap.get(type);
        if (convertor == null) {
            throw new RuntimeException("Cannot get converter:" + type.getName());
        }
        return convertor;
    }

    public static <T> T convert(Class<T> targetType, Class<?> variableType, Object variable) {
        BasicTypeConvertor convertor = getConvertor(targetType);
        Object convert = convertor.convert(variableType, variable);
        return (T) convert;
    }
}
