package org.brynhild.graduation.common.utils;

import org.apache.commons.io.FileUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;

import java.io.*;
import java.nio.charset.StandardCharsets;

public class ByteTransferUtil {
    private ByteTransferUtil() {
    }

    public static byte[] getBytesFromObject(Serializable obj) throws Exception {
        if (obj == null) {
            return null;
        }
        ByteArrayOutputStream bo = new ByteArrayOutputStream();
        ObjectOutputStream oo = new ObjectOutputStream(bo);
        oo.writeObject(obj);
        return bo.toByteArray();
    }

    public static Object getObjectFromBytes(byte[] objBytes) throws Exception {
        if (objBytes == null || objBytes.length == 0) {
            return null;
        }
        ByteArrayInputStream bi = new ByteArrayInputStream(objBytes);
        ObjectInputStream oi = new ObjectInputStream(bi);
        return oi.readObject();
    }

    public static ResponseEntity<byte[]> outputDataToResponse(File file) throws IOException {
        HttpHeaders headers = new HttpHeaders();
        String downloadFileName = new String(file.getName().getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
        headers.setContentDispositionFormData("attachment", downloadFileName);
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        return new ResponseEntity<byte[]>(FileUtils.readFileToByteArray(file), headers, HttpStatus.CREATED);
    }

    public static byte[] int2Byte(int intValue, boolean reverse) {
        byte[] b = new byte[4];
        for (int i = 0; i < 4; i++) {
            b[i] = (byte) (intValue >> 8 * (3 - i) & 0xFF);
        }
        if (reverse) {
            for (int i = 0; i < 2; i++) {
                byte tmp = b[i];
                b[i] = b[4 - 1 - i];
                b[4 - 1 - i] = tmp;
            }
        }
        return b;
    }

    public static byte[] int2Byte(int intValue) {
        return int2Byte(intValue, false);
    }

    public static int byte2Int(byte[] b, boolean reverse) {
        if (b == null || b.length != 4) {
            throw new IllegalArgumentException("byte array is NULL or length mismatch!");
        }
        if (reverse) {
            for (int i = 0; i < 2; i++) {
                byte tmp = b[i];
                b[i] = b[4 - 1 - i];
                b[4 - 1 - i] = tmp;
            }
        }
        int intValue = 0;
        for (int i = 0; i < b.length; i++) {
            intValue += (b[i] & 0xFF) << (8 * (3 - i));
        }
        return intValue;
    }

    public static String bytesToString(byte[] bytes, String separator) {
        if (bytes == null) {
            return "";
        }
        StringBuilder builder = new StringBuilder();
        boolean first = true;
        for (byte b : bytes) {
            if (!first) {
                builder.append(separator);
            } else {
                first = false;
            }
            builder.append(String.valueOf(b));
        }
        return builder.toString();
    }

    public static void main(String[] args) {
    }

    public static int byte2Int(byte[] b) {
        return byte2Int(b, false);
    }


}
