package org.brynhild.graduation.common.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import jakarta.annotation.PostConstruct;
import lombok.Data;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

@Component
@ConfigurationProperties("org.brynhild.graduation.jwt")
@Data
public class JwtUtil {
    private String key;
    private long ttl;

    @PostConstruct
    public void init() {
        System.out.println("[INFO]JWTUtil init with key: " + key + "   ttl: " + ttl);
    }

    public String createJwt(Map<String, Object> claims) {
        long nowMillis = System.currentTimeMillis();
        Date now = new Date(nowMillis);
        JwtBuilder builder = Jwts.builder()
                .setIssuedAt(now)
                .signWith(SignatureAlgorithm.HS256, key)
                .setClaims(claims);
        if (ttl > 0) {
            builder.setExpiration(new Date(nowMillis + ttl));
        }
        return builder.compact();
    }

    public Claims parseJWT(String jwtStr) {
        return Jwts.parser().setSigningKey(key)
                .parseClaimsJws(jwtStr)
                .getBody();
    }

    public Long getId(String token) {
        return Long.parseLong(parseJWT(token).get(AccountConstant.TOKEN_ID, String.class));
    }
}
