package org.brynhild.graduation.common.utils;


import java.util.Arrays;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 求和、取平均、包含、部分包含、全部包含
 * 求和取平均重载参数：big：使用BigInteger或BigDecimal处理数据
 * 包含、部分包含、全部包含重载参数：hash：使用HashSet进行匹配
 */
public class NumberUtil {
    private NumberUtil() {
    }

    public static Long getLongSum(List<Long> nums) {
        if (nums == null || nums.isEmpty()) {
            return 0L;
        }
        AtomicReference<Long> sum = new AtomicReference<>(0L);
        nums.forEach(num -> sum.updateAndGet(v -> v + num));
        return sum.get();
    }

    public static Integer getIntegerSum(List<Integer> nums) {
        if (nums == null || nums.isEmpty()) {
            return 0;
        }
        AtomicReference<Integer> sum = new AtomicReference<>(0);
        nums.forEach(num -> sum.updateAndGet(v -> v + num));
        return sum.get();
    }

    public static Float getFloatSum(List<Float> nums) {
        if (nums == null || nums.isEmpty()) {
            return 0.0f;
        }
        AtomicReference<Float> sum = new AtomicReference<>(0.0f);
        nums.forEach(num -> sum.updateAndGet(v -> v + num));
        return sum.get();
    }

    public static Double getDoubleSum(List<Double> nums) {
        if (nums == null || nums.isEmpty()) {
            return 0.0;
        }
        AtomicReference<Double> sum = new AtomicReference<>(0.0);
        nums.forEach(num -> sum.updateAndGet(v -> v + num));
        return sum.get();
    }

    public static Long geLongSum(Long[] nums) {
        if (nums == null || nums.length == 0) {
            return 0L;
        }
        return getLongSum(Arrays.asList(nums));
    }

    public static Integer getIntegerSum(Integer[] nums) {
        if (nums == null || nums.length == 0) {
            return 0;
        }
        return getIntegerSum(Arrays.asList(nums));
    }

    public static Float getFloatSum(Float[] nums) {
        if (nums == null || nums.length == 0) {
            return 0.0f;
        }
        return getFloatSum(Arrays.asList(nums));
    }

    public static Double getDoubleSum(Double[] nums) {
        if (nums == null || nums.length == 0) {
            return 0.0;
        }
        return getDoubleSum(Arrays.asList(nums));
    }

    public static boolean contains(Long target, List<Long> nums) {
        if (target == null || nums == null) {
            return false;
        }
        return nums.contains(target);
    }

    public static boolean contains(Long target, Long[] nums) {
        if (target == null || nums == null) {
            return false;
        }
        return contains(target, Arrays.asList(nums));
    }

    public static boolean contains(Integer target, List<Integer> nums) {
        if (target == null || nums == null) {
            return false;
        }
        return nums.contains(target);
    }

    public static boolean contains(Integer target, Integer[] nums) {
        if (target == null || nums == null) {
            return false;
        }
        return contains(target, Arrays.asList(nums));
    }

    public static boolean contains(Float target, List<Float> nums) {
        if (target == null || nums == null) {
            return false;
        }
        return nums.contains(target);
    }

    public static boolean contains(Float target, Float[] nums) {
        if (target == null || nums == null) {
            return false;
        }
        return contains(target, Arrays.asList(nums));
    }

    public static boolean contains(Double target, List<Double> nums) {
        if (target == null || nums == null) {
            return false;
        }
        return nums.contains(target);
    }

    public static boolean contains(Double target, Double[] nums) {
        if (target == null || nums == null) {
            return false;
        }
        return contains(target, Arrays.asList(nums));
    }

    //return true if target could be found in B
    private static void contains() {

    }

    //return true if all element in A could be found in B
    private static void containsAll() {

    }

    //return the count that element in A could be found in B
    private static void matchCount() {

    }

    //return true if exist at least one element in A could be found in B
    //parameter hash:boolean->false:use List    true:use HashMap
    //reload function:hash is true
    private static void containsAny() {

    }


}
