package org.brynhild.graduation.common.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TypeCastUtil {

    public static <K, V> Map<K, V> castToMap(Object object, Class<K> keyType, Class<V> valueType) {

        if (object == null) {
            return null;
        }
        if (object instanceof Map<?, ?>) {
            Map<?, ?> tmp = (Map<?, ?>) object;
            Map<K, V> ret = new HashMap<>();
            for (Map.Entry<?, ?> entry : tmp.entrySet()) {
                ret.put(keyType.cast(entry.getKey()), valueType.cast(entry.getValue()));
            }
            return ret;
        }
        throw new ClassCastException("Type expected: Map<" + keyType.getName() + "," + valueType.getName() + "> ,actual: " + object.getClass().getName());
    }

    public static <T> List<T> castToList(Object object, Class<T> type) {
        if (object == null) {
            return null;
        }
        if (object instanceof List<?>) {
            List<?> tmp = (List<?>) object;
            List<T> ret = new ArrayList<>();
            for (Object item : tmp) {
                ret.add(type.cast(item));
            }
            return ret;
        }
        throw new ClassCastException("Type expected: List<" + type.getName() + "> ,actual: " + object.getClass().getName());
    }

    public static List<Object> castToList(Object object) {
        if (object == null) {
            return null;
        }
        if (object instanceof List<?>) {
            List<?> tmp = (List<?>) object;
            return new ArrayList<>(tmp);
        }
        throw new ClassCastException("Type expected: List<Object> ,actual: " + object.getClass().getName());
    }

    public static <T> Class<? extends T> castToSubClass(Object object, Class<T> type) {
        return ((Class<?>) object).asSubclass(type);
    }

    public static <T> T castToObject(Object object, Class<T> type) {
        if (object == null) {
            return null;
        }
        return type.cast(object);
    }

}
