package org.brynhild.graduation.device.controller


import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.RequestBodyVerifier
import org.brynhild.graduation.device.service.IAdminService
import org.brynhild.graduation.transfer.device.vo.*
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.validation.BindingResult
import org.springframework.validation.annotation.Validated
import org.springframework.web.bind.annotation.*

@RequestMapping("/admin")
@RestController
class AdminController {
    private IAdminService adminService
    private RequestBodyVerifier verifier

    @PostMapping("/device/bind")
    Result addDevice(@RequestBody @Validated AdminAddDevice request, BindingResult result,
                     @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.addDevice(request, token)
    }

    @PostMapping("/device/enroll")
    Result enrollFingerPrint(@RequestBody @Validated EnrollFingerPrint info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.enrollFingerPrint(info, token)
    }

    @PostMapping("/device")
    Result findDevice(@RequestBody @Validated QueryDeviceRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.findDevice(request)
    }

    @PutMapping("/device")
    Result modifyDevice(@RequestBody @Validated AdminModifyDevice info, BindingResult result,
                        @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.modifyDevice(info, token)
    }

    @DeleteMapping("/device")
    Result deleteDevice(@RequestBody @Validated AdminDeleteDevice info, BindingResult result,
                        @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.deleteDevice(info, token)
    }

    @PostMapping("/area/add")
    Result addArea(@RequestBody @Validated AdminAddArea info, BindingResult result,
                   @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.addArea(info, token)
    }

    @PostMapping("/area")
    Result findArea(@RequestBody @Validated QueryAreaRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.findArea(request)
    }

    @PutMapping("/area")
    Result modifyArea(@RequestBody @Validated AdminModifyArea info, BindingResult result,
                      @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.modifyArea(info, token)
    }

    @DeleteMapping("/area")
    Result deleteArea(@RequestBody @Validated AdminDeleteArea info, BindingResult result,
                      @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.deleteArea(info, token)
    }

    @PostMapping("/credential/add")
    Result addCredential(@RequestBody @Validated AdminAddCredential info, BindingResult result,
                         @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.addCredential(info, token)
    }

    @PostMapping("/credential")
    Result findCredential(@RequestBody @Validated QueryCredentialRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.findCredential(request)
    }

    @DeleteMapping("/credential")
    Result deleteCredential(@RequestBody @Validated AdminDeleteCredential info, BindingResult result,
                            @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.deleteCredential(info, token)
    }

    @PostMapping("/signInRecord")
    Result findSignInRecord(@RequestBody @Validated QuerySignInRecordRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.findSignInRecord(request)
    }

    @PostMapping("/relationship/add")
    Result addRelationship(@RequestBody @Validated AdminAddRelationship info, BindingResult result,
                           @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.addRelationship(info, token)
    }

    @PostMapping("/relationship")
    Result findRelationship(@RequestBody @Validated QueryRelationshipRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.findRelationship(request)
    }

    @DeleteMapping("/relationship")
    Result deleteRelationship(@RequestBody @Validated AdminDeleteRelationship info, BindingResult result,
                              @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.deleteRelationship(info, token)
    }

    @PostMapping("/batch/finger")
    Result getClassFingerPrintDataIdList(@RequestBody @Validated AdminGetClassFingerPrintDataIdList info, BindingResult result,
                                         @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.getClassFingerPrintDataIdList(info, token)
    }

    @PostMapping("/batch/clear")
    Result clearFingerPrintDataAtDevice(@RequestBody @Validated AdminClearFingerPrintDataAtDevice info, BindingResult result,
                                        @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.clearFingerPrintDataAtDevice(info, token)
    }

    @PostMapping("/batch/push")
    Result pushFingerPrintDataToDevice(@RequestBody @Validated AdminPushFingerPrintDataToDevice info, BindingResult result,
                                       @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.pushFingerPrintDataToDevice(info, token)
    }

    @PostMapping("/batch/commit")
    Result commitPush(@RequestBody @Validated AdminCommitPush info, BindingResult result, @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result)
        }
        return adminService.commitPush(info, token)
    }


    @Autowired
    @Resource
    void setVerifier(RequestBodyVerifier verifier) {
        this.verifier = verifier
    }

    @Autowired
    @Resource
    void setAdminService(IAdminService adminService) {
        this.adminService = adminService
    }
}
