package org.brynhild.graduation.device.controller

import jakarta.annotation.Resource
import org.brynhild.graduation.common.utils.RequestBodyVerifier
import org.brynhild.graduation.device.service.IDeviceService
import org.brynhild.graduation.transfer.device.bo.AuthenticRequest
import org.brynhild.graduation.transfer.device.bo.HeartBeatRequest
import org.brynhild.graduation.transfer.device.bo.RegisterRequest
import org.brynhild.graduation.transfer.device.bo.RegisterResponse
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.validation.BindingResult
import org.springframework.validation.annotation.Validated
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RequestMapping("/device")
@RestController
class DeviceController {
    private IDeviceService deviceService
    private RequestBodyVerifier verifier

    @PostMapping("/authentic")
    Integer authentic(@RequestBody @Validated AuthenticRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return AuthenticRequest.hasError()
        }
        return deviceService.authentic(request)
    }

    @PostMapping("/heartbeat")
    Integer heartbeat(@RequestBody @Validated HeartBeatRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return HeartBeatRequest.hasError()
        }
        return deviceService.heartbeat(request)
    }


    @PostMapping("/register")
    public RegisterResponse register(@RequestBody @Validated RegisterRequest request, BindingResult result) {
        if (result.hasErrors()) {
            return new RegisterResponse(0, "000000")
        }
        return deviceService.register(request)
    }

    @Autowired
    @Resource
    void setDeviceService(IDeviceService deviceService) {
        this.deviceService = deviceService
    }

    @Autowired
    @Resource
    void setVerifier(RequestBodyVerifier verifier) {
        this.verifier = verifier
    }
}
