package org.brynhild.graduation.device.task

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.DeviceConstant
import org.brynhild.graduation.device.event.MQTTHandler
import org.brynhild.graduation.device.model.DeviceAndTimeTableMapping
import org.brynhild.graduation.persistence.device.repository.DeviceRepository
import org.brynhild.graduation.persistence.timetable.entity.TimeTable
import org.brynhild.graduation.persistence.timetable.repository.TimeTableRepository
import org.brynhild.graduation.transfer.view.ViewConvertor
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.scheduling.annotation.EnableScheduling
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component

@Component
@EnableScheduling
class PrimaryPusher {

    private MQTTHandler handler
    private TimeTableRepository tableRepository
    private DeviceRepository deviceRepository
    private ViewConvertor viewConvertor

    @Scheduled(fixedRate = 1800000)
    void pushToPrimaryDevice() {
        def listOfTables = tableRepository.findTimeTableByPushed(false)
        Map<Long, List<TimeTable>> areaTimeTableMap = new HashMap<>()
        for (table in listOfTables) {
            areaTimeTableMap.computeIfAbsent(table.getAreaId(), v -> new ArrayList<>())
            areaTimeTableMap.get(table.getAreaId()).add(table)
        }
        def areaIdList = areaTimeTableMap.keySet()
        List<DeviceAndTimeTableMapping> terminalMapping = new ArrayList<>();
        for (areaId in areaIdList) {
            def listOfDevices = deviceRepository.findDeviceByAreaAndType(areaId, DeviceConstant.TERMINAL)
            if (listOfDevices.size() == 1) {
                def device = listOfDevices.get(0)
                terminalMapping.add(new DeviceAndTimeTableMapping(device, areaTimeTableMap.get(areaId)))
            }
        }

        Map<Long, List<List<TimeTable>>> secondaryGatewayMapping = new HashMap<>()
        for (mapping in terminalMapping) {
            def master = mapping.device.getMaster()
            if (master == null) {
                continue
            }
            def optional = deviceRepository.findById(master)
            optional.ifPresent {
                secondaryGatewayMapping.computeIfAbsent(master, lsit -> new ArrayList<>())
                secondaryGatewayMapping.get(master).add(mapping.timeTableList)
            }
        }

        Map<Long, List<List<List<TimeTable>>>> primaryGatewayMapping = new HashMap<>()
//        for(mapping in secondaryGatewayMapping){
//
//        }


    }

    @Scheduled(fixedRate = 1800000)
    void pushToDeviceDirectly() {
        def listOfTables = tableRepository.findTimeTableByPushed(false)
        Map<Long, List<TimeTable>> areaTimeTableMap = new HashMap<>()
        for (table in listOfTables) {
            areaTimeTableMap.computeIfAbsent(table.getAreaId(), v -> new ArrayList<>())
            areaTimeTableMap.get(table.getAreaId()).add(table)
        }
        def areaIdList = areaTimeTableMap.keySet()
        List<DeviceAndTimeTableMapping> terminalMapping = new ArrayList<>()
        for (areaId in areaIdList) {
            def listOfDevices = deviceRepository.findDeviceByAreaAndType(areaId, DeviceConstant.TERMINAL)
            if (listOfDevices.size() == 1) {
                def device = listOfDevices.get(0)
                terminalMapping.add(new DeviceAndTimeTableMapping(device, areaTimeTableMap.get(areaId)))
            }
        }
        for (mapping in terminalMapping) {
            def device = mapping.device
            def timetableList = mapping.timeTableList

            for (table in timetableList) {
                def timeTableBinaries = viewConvertor.convertToTimetableBinary(table)
                boolean finished = true
                for (binary in timeTableBinaries) {
                    def toDevice = handler.sendMessageToDevice(device.getMac(), binary.convertToBytes())
                    if (!toDevice) {
                        finished = toDevice
                    }
                }
                if (finished) {
                    table.setPushed(true)
                    tableRepository.save(table)
                }
            }
        }
    }

    @Autowired
    @Resource
    void setHandler(MQTTHandler handler) {
        this.handler = handler
    }

    @Autowired
    @Resource
    void setTableRepository(TimeTableRepository tableRepository) {
        this.tableRepository = tableRepository
    }

    @Autowired
    @Resource
    void setDeviceRepository(DeviceRepository deviceRepository) {
        this.deviceRepository = deviceRepository
    }

    @Autowired
    @Resource
    void setViewConvertor(ViewConvertor viewConvertor) {
        this.viewConvertor = viewConvertor
    }
}
