package org.brynhild.graduation.event.listener

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.event.service.IEmailService
import org.brynhild.graduation.transfer.user.bo.SendActiveEmailEvent
import org.brynhild.graduation.transfer.user.bo.SendResetEmailEvent
import org.springframework.amqp.rabbit.annotation.RabbitHandler
import org.springframework.amqp.rabbit.annotation.RabbitListener
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
@RabbitListener(queues = RabbitQueueConstant.EMAIL)
class EmailListener {
    private IEmailService emailService

    @RabbitHandler
    void sendEmail(byte[] bytes) {
        try {
            Object objectFromBytes = ByteTransferUtil.getObjectFromBytes(bytes)
            switch (objectFromBytes) {
                case SendActiveEmailEvent:
                    emailService.sendActiveEmail(objectFromBytes)
                    println "active"
                    break
                case SendResetEmailEvent:
                    emailService.sendResetEmail(objectFromBytes)
                    println "reset"
                    break
                default: throw new IllegalStateException("Unexpected value: " + objectFromBytes)
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }

    @Autowired
    @Resource
    void setEmailService(IEmailService emailService) {
        this.emailService = emailService
    }
}
