package org.brynhild.graduation.event.listener

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.event.service.ILogService
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.springframework.amqp.rabbit.annotation.RabbitHandler
import org.springframework.amqp.rabbit.annotation.RabbitListener
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
@RabbitListener(queues = RabbitQueueConstant.OPERATION_LOG)
class OperationLogListener {
    private ILogService logService

    @RabbitHandler
    void handleLog(byte[] bytes) {
        try {
            Object object = ByteTransferUtil.getObjectFromBytes(bytes)
            switch (object) {
                case OperationLog:
                    logService.addOperationLog(object)
                    break
                default: throw new IllegalStateException("Unexpected value: " + object)
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }

    @Autowired
    @Resource
    void setLogService(ILogService logService) {
        this.logService = logService
    }
}
