package org.brynhild.graduation.event.listener

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.event.service.IUserService
import org.brynhild.graduation.transfer.user.bo.UserOfflineEvent
import org.brynhild.graduation.transfer.user.bo.UserSuccessLoginEvent
import org.springframework.amqp.rabbit.annotation.RabbitHandler
import org.springframework.amqp.rabbit.annotation.RabbitListener
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
@RabbitListener(queues = RabbitQueueConstant.USER)
class UserListener {
    private IUserService userService

    @RabbitHandler
    void handle(byte[] bytes) {
        try {
            Object objectFromBytes = ByteTransferUtil.getObjectFromBytes(bytes)
            switch (objectFromBytes) {
                case UserSuccessLoginEvent:
                    userService.replaceToken(objectFromBytes)
                    break
                case UserOfflineEvent:
                    userService.offline(objectFromBytes)
                    break
                default: throw new IllegalStateException("Unexpected value: " + objectFromBytes)
            }
        } catch (Exception e) {
            e.printStackTrace()
        }
    }

    @Autowired
    @Resource
    void setUserService(IUserService userService) {
        this.userService = userService
    }
}
