package org.brynhild.graduation.event.service.impl


import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.constant.RedisConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.event.service.IUserService
import org.brynhild.graduation.transfer.event.bo.GatewayDeleteToken
import org.brynhild.graduation.transfer.event.bo.GatewayUpdateToken
import org.brynhild.graduation.transfer.user.bo.UserOfflineEvent
import org.brynhild.graduation.transfer.user.bo.UserSuccessLoginEvent
import org.springframework.amqp.rabbit.core.RabbitTemplate
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.stereotype.Service

import java.util.concurrent.TimeUnit

@Service
class UserServiceImpl implements IUserService {
    private RedisTemplate<String, Object> redisTemplate
    private RabbitTemplate rabbitTemplate

    @Override
    void replaceToken(UserSuccessLoginEvent info) {
        redisTemplate.opsForValue().set(RedisConstant.USER_ACTIVE_TOKEN + info.username, info.token, 1, TimeUnit.DAYS)
        rabbitTemplate.convertAndSend(RabbitQueueConstant.USER_TOKEN, ByteTransferUtil.getBytesFromObject(new GatewayUpdateToken(info.username, info.token)));
    }

    @Override
    void offline(UserOfflineEvent info) {
        rabbitTemplate.convertAndSend(RabbitQueueConstant.USER_TOKEN, ByteTransferUtil.getBytesFromObject(new GatewayDeleteToken(info.username)))
        redisTemplate.delete(RedisConstant.USER_ACTIVE_TOKEN + info.username)
    }

    @Autowired
    @Resource
    void setRedisTemplate(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate
    }

    @Autowired
    @Resource
    void setRabbitTemplate(RabbitTemplate rabbitTemplate) {
        this.rabbitTemplate = rabbitTemplate
    }
}
