package org.brynhild.graduation.file.service.impl;


import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.ByteTransferUtil;
import org.brynhild.graduation.common.utils.IdWorker;
import org.brynhild.graduation.file.config.CosConfig;
import org.brynhild.graduation.file.executor.CosJavaRepository;
import org.brynhild.graduation.file.service.IFileService;
import org.brynhild.graduation.persistence.file.entity.LocalFile;
import org.brynhild.graduation.persistence.file.repository.FileRepository;
import org.brynhild.graduation.transfer.file.dto.FileInfo;
import org.springframework.http.HttpEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.PostConstruct;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class FileServiceImpl implements IFileService {

    private final FileRepository fileRepository;

    private final CosJavaRepository cosRepository;

    private final CosConfig config;

    private final IdWorker idWorker;

    private boolean windows = true;
    private String rootPath;

    @PostConstruct
    public void initPath() {
        windows = System.getProperty("os.name").toLowerCase().startsWith("win");
        if (windows) {
            rootPath = "C:\\upload";
        } else {
            rootPath = "/home/orange/jars/upload";
        }
    }

    @Override
    public Result fileUpload(MultipartFile[] files, HttpServletRequest request) {
        List<FileInfo> idList = new ArrayList<>();
        for (MultipartFile file : files) {
            String origin = file.getOriginalFilename();
            assert origin != null;
            String filename = idWorker.nextId() + "" + origin.substring(origin.lastIndexOf("."));

            System.out.println("PhysicalPath:" + rootPath + File.separator + filename);
            File targetFile = new File(rootPath + File.separator + filename);
            LocalFile localFile = new LocalFile();
            localFile.setPath(filename);
            try {
                if (!targetFile.exists()) {
                    targetFile.createNewFile();
                }
                file.transferTo(targetFile);
                localFile = fileRepository.save(localFile);
                FileInfo fileInfo = new FileInfo();
                fileInfo.setId(localFile.getId());
                fileInfo.setPath(config.getRedirectPath() + localFile.getPath());
                idList.add(fileInfo);
                System.out.println("Service添加任务");
                cosRepository.addUploadFileTask(targetFile, localFile);
            } catch (Exception e) {
                e.printStackTrace();
                return new Result(false, "文件上传失败");
            }
        }
        return new Result(true, "文件上传成功", idList);
    }

    @Override
    public Result singleFileUpload(MultipartFile file, HttpServletRequest request) {
        String origin = file.getOriginalFilename();
        assert origin != null;
        String filename = idWorker.nextId() + "" + origin.substring(origin.lastIndexOf("."));

        System.out.println("PhysicalPath:" + rootPath + File.separator + filename);
        File targetFile = new File(rootPath + File.separator + filename);
        LocalFile localFile = new LocalFile();
        localFile.setPath(filename);
        try {
            if (!targetFile.exists()) {
                targetFile.createNewFile();
            }
            file.transferTo(targetFile);
            localFile = fileRepository.save(localFile);
            System.out.println("Service添加任务");
            cosRepository.addUploadFileTask(targetFile, localFile);
            List<FileInfo> idList = new ArrayList<>();
            FileInfo fileInfo = new FileInfo();
            fileInfo.setId(localFile.getId());
            fileInfo.setPath(config.getRedirectPath() + localFile.getPath());
            System.out.println(fileInfo.getPath());
            idList.add(fileInfo);
            return new Result(true, "文件上传成功", idList);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, "文件上传失败");
        }
    }

    @Override
    public Result singleFileUploadSync(MultipartFile file, HttpServletRequest request) {
        String origin = file.getOriginalFilename();
        assert origin != null;
        String filename = idWorker.nextId() + "" + origin.substring(origin.lastIndexOf("."));

        System.out.println("PhysicalPath:" + rootPath + File.separator + filename);
        File targetFile = new File(rootPath + File.separator + filename);
        LocalFile localFile = new LocalFile();
        localFile.setPath(filename);
        try {
            if (!targetFile.exists()) {
                targetFile.createNewFile();
            }
            file.transferTo(targetFile);
            localFile = fileRepository.save(localFile);
            System.out.println("Service添加同步任务");
            cosRepository.addUploadFileTaskSync(targetFile, localFile);
            List<FileInfo> idList = new ArrayList<>();
            FileInfo fileInfo = new FileInfo();
            fileInfo.setId(localFile.getId());
            fileInfo.setPath(config.getRedirectPath() + localFile.getPath());
            System.out.println(fileInfo.getPath());
            idList.add(fileInfo);
            return new Result(true, "文件上传成功", idList);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, "文件上传失败");
        }
    }

    @Override
    public HttpEntity<byte[]> fileDownloadByName(String name, HttpServletResponse response) {
        final LocalFile localFile = fileRepository.findLocalFileByPath(name);
        if (localFile != null) {
            if (localFile.getUpload()) {
                try {
                    response.sendRedirect(config.getRedirectPath() + name);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return null;
            }
        }
        File file = new File(rootPath + File.separator + name);
        try {
            return ByteTransferUtil.outputDataToResponse(file);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public HttpEntity<byte[]> fileDownloadById(Long id, HttpServletResponse response) {
        Optional<LocalFile> fileOptional = fileRepository.findById(id);
        if (fileOptional.isEmpty()) {
            return null;
        }
        LocalFile localFile = fileOptional.get();
        if (localFile.getUpload()) {
            try {
                response.sendRedirect(config.getRedirectPath() + localFile.getPath());
            } catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }
        String path = rootPath + File.separator + localFile.getPath();
        File file = new File(path);
        try {
            return ByteTransferUtil.outputDataToResponse(file);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    public Result fileInfo(Long id) {
        Optional<LocalFile> fileOptional = fileRepository.findById(id);
        if (fileOptional.isEmpty()) {
            return new Result(false, "没有这个文件");
        }
        LocalFile localFile = fileOptional.get();
        FileInfo fileInfo = new FileInfo();
        fileInfo.setId(localFile.getId());
        fileInfo.setPath(localFile.getPath());
        return new Result(true, "查询文件成功", fileInfo);
    }
}
