package org.brynhild.graduation.gateway.route;

import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.gateway.rule.ApiRule;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.route.RouteLocator;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.core.Ordered;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

@Component
@RequiredArgsConstructor
public class ApplicationGlobalFilter implements GatewayFilter, Ordered {

    private final ApiRule publicApiRule;

    private final ApiRule tokenValidRule;

    private final ApiRule authorityRule;


    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpResponse response = exchange.getResponse();

        if (publicApiRule.permit(exchange)) {
            return chain.filter(exchange);
        }
        if (!tokenValidRule.permit(exchange)) {
            System.out.println("invalid token");
            response.setStatusCode(HttpStatus.NON_AUTHORITATIVE_INFORMATION);
            return response.setComplete();
        }
        if (!authorityRule.permit(exchange)) {
            System.out.println("unauthorized");
            response.setStatusCode(HttpStatus.UNAUTHORIZED);
            return response.setComplete();
        }
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }


    @Bean
    public RouteLocator route(RouteLocatorBuilder builder) {
        return builder.routes()
                .route(p -> p
                        .path("/user/**")
                        .filters(f -> f
                                .filter(this)
                                .stripPrefix(1))
                        .uri("http://localhost:8001"))
                .route(p -> p
                        .path("/event/**")
                        .filters(f -> f.filter(this)
                                .stripPrefix(1))
                        .uri("http://20.204.135.132:8002"))
                .route(p -> p
                        .path("/file/**")
                        .filters(f -> f.filter(this)
                                .stripPrefix(1))
                        .uri("http://localhost:8003"))
                .route(p -> p
                        .path("/device/**")
                        .filters(f -> f.filter(this)
                                .stripPrefix(1))
                        .uri("http://localhost:8004"))
                .route(p -> p
                        .path("/timetable/**")
                        .filters(f -> f.filter(this)
                                .stripPrefix(1))
                        .uri("http://localhost:8005"))
                .build();
    }
}
