package org.brynhild.graduation.gateway.rule


import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.gateway.rule.config.AuthorityMap
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpHeaders
import org.springframework.http.server.reactive.ServerHttpRequest
import org.springframework.stereotype.Component
import org.springframework.web.server.ServerWebExchange

@Component
class AuthorityRule implements ApiRule {
    private JwtUtil jwtUtil
    private AuthorityMap authorityMap

    @Override
    boolean permit(ServerWebExchange exchange) {
        ServerHttpRequest request = exchange.getRequest()
        HttpHeaders headers = request.getHeaders()
        String path = request.getPath().toString()
        println "visit:${path}"
        try {
            def claims = jwtUtil.parseJWT(headers.getFirst(AccountConstant.ACCOUNT_HEADER))
            def role = claims.get(AccountConstant.TOKEN_ROLE) as String
            if (role != null) {
                role = Integer.parseInt(role)
                path = StringUtil.stripPrefix(path, 1)
                println "visit path in app:${path},role:${role}"
                return authorityMap.permit(path, role)
            } else {
                return false
            }
        } catch (Exception e) {
            e.printStackTrace()
            return false
        }
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setAuthorityMap(AuthorityMap authorityMap) {
        this.authorityMap = authorityMap
    }
}
