package org.brynhild.graduation.persistence.common.strategy;

import org.brynhild.graduation.persistence.common.constant.CommonDBStringDefinition;
import org.hibernate.boot.model.naming.Identifier;
import org.hibernate.boot.model.naming.PhysicalNamingStrategyStandardImpl;
import org.hibernate.cfg.ImprovedNamingStrategy;
import org.hibernate.engine.jdbc.env.spi.JdbcEnvironment;

import java.util.Arrays;
import java.util.List;

public abstract class BaseDBNamingStrategy extends PhysicalNamingStrategyStandardImpl {
    private static final List<String> ignoreColumnNameList = Arrays.asList(
            CommonDBStringDefinition.COLUMN_NAME_ID,
            CommonDBStringDefinition.COLUMN_NAME_DELETE,
            CommonDBStringDefinition.COLUMN_NAME_VERSION
    );

    public abstract String getTablePrefix();

    public abstract String getColumnPrefix();

    @Override
    public Identifier toPhysicalTableName(Identifier identifier, JdbcEnvironment jdbcEnvironment) {
        return Identifier.toIdentifier(getTablePrefix() +
                ImprovedNamingStrategy.INSTANCE.classToTableName(identifier.toString()).toUpperCase());
    }

    @Override
    public Identifier toPhysicalColumnName(Identifier identifier, JdbcEnvironment jdbcEnvironment) {
        String tableName = ImprovedNamingStrategy.INSTANCE.classToTableName(identifier.toString()).toUpperCase();
        return Identifier.toIdentifier(
                (ignoreColumnNameList.contains(tableName) ? "" : getColumnPrefix()) + tableName);
    }
}
