package org.brynhild.graduation.persistence.timetable.entity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.Table;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.brynhild.graduation.common.tag.LogFieldDescription;
import org.brynhild.graduation.persistence.common.constant.CommonDBStringDefinition;
import org.brynhild.graduation.persistence.common.entity.LogicDeleteEntity;
import org.hibernate.annotations.Where;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.List;

@Data
@Table(name = "timetable")
@Entity
@EqualsAndHashCode(callSuper = true)
@Where(clause = CommonDBStringDefinition.LOGIC_DELETE_WHERE_CLAUSE)
@NoArgsConstructor
public class TimeTable extends LogicDeleteEntity implements Serializable {
    private static List<String> startSignInTime = Arrays.asList("07:50:00", "08:45:00", "10:00:00", "10:55:00", "13:50:00", "14:45:00", "16:00:00", "16:55:00", "18:50:00", "19:45:00", "20:40:00");
    private static List<String> endSignInTime = Arrays.asList("08:10:00", "09:05:00", "10:20:00", "11:15:00", "14:10:00", "15:05:00", "16:20:00", "17:15:00", "19:10:00", "20:05:00", "21:00:00");
    private static List<String> startSignOutTime = Arrays.asList("08:35:00", "09:30:00", "10:45:00", "11:40:00", "14:35:00", "15:30:00", "16:45:00", "17:40:00", "19:35:00", "20:30:00", "21:35:00");
    private static List<String> endSignOutTime = Arrays.asList("08:55:00", "09:50:00", "11:05:00", "12:00:00", "14:55:00", "15:50:00", "17:05:00", "18:00:00", "19:55:00", "20:50:00", "21:55:00");
    @LogFieldDescription("区域id")
    @Column(name = "area_id")
    private Long areaId;
    @LogFieldDescription("时间点")
    @Column(name = "_index")
    private Integer index;
    @LogFieldDescription("节数")
    private Integer step;
    @LogFieldDescription("周几")
    private Integer day;
    @LogFieldDescription("教师")
    private Long teacher;
    @LogFieldDescription("周次")
    private String week;
    @LogFieldDescription("班级列表")
    private String clazz;
    @LogFieldDescription("课程id")
    @Column(name = "course_id")
    private Long courseId;
    @Column(name = "course_name")
    private String courseName;
    @LogFieldDescription("年份")
    private Integer year;
    @LogFieldDescription("学期")
    private Integer term;
    @LogFieldDescription("是否推送")
    private Boolean pushed = false;

    public String getStartSignInTime() {
        LocalDate start = LocalDate.of(2023, 2, 20);
        start = start.plusWeeks(Long.parseLong(week) - 1);
        start = start.plusDays(day.longValue() - 1);
        String date = start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        return date + " " + startSignInTime.get(index - 1);
    }

    public String getEndSignInTime() {
        LocalDate start = LocalDate.of(2023, 2, 20);
        start = start.plusWeeks(Long.parseLong(week) - 1);
        start = start.plusDays(day.longValue() - 1);
        String date = start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        return date + " " + endSignInTime.get(index - 1);
    }

    public String getStartSignOutTime() {
        LocalDate start = LocalDate.of(2023, 2, 20);
        start = start.plusWeeks(Long.parseLong(week) - 1);
        start = start.plusDays(day.longValue() - 1);
        String date = start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        return date + " " + startSignOutTime.get(index + step - 2);
    }

    public String getEndSignOutTime() {
        LocalDate start = LocalDate.of(2023, 2, 20);
        start = start.plusWeeks(Long.parseLong(week) - 1);
        start = start.plusDays(day.longValue() - 1);
        String date = start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        return date + " " + endSignOutTime.get(index + step - 2);
    }
}

