package org.brynhild.graduation.timetable.controller;

import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.brynhild.graduation.common.parser.LogGenerator;
import org.brynhild.graduation.common.parser.SpecificationGenerator;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.RequestBodyVerifier;
import org.brynhild.graduation.timetable.service.ICourseService;
import org.brynhild.graduation.timetable.service.ITimeTableService;
import org.brynhild.graduation.transfer.timetable.vo.*;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.File;

@RestController
@RequestMapping("/admin")
@RequiredArgsConstructor
public class AdminController {
    private final RequestBodyVerifier verifier;
    private final ITimeTableService tableService;
    private final ICourseService courseService;

    @PostConstruct
    public void init() {
        String tip = LogGenerator.generate(new File("logGenerate.groovy"), "org.brynhild.graduation.transfer.timetable") ? "ok" : "error";
        System.out.println("generate log:" + tip);
        tip = SpecificationGenerator.generate(new File("specificationGenerate.groovy"), "org.brynhild.graduation.transfer.timetable") ? "ok" : "error";
        System.out.println("generate specification:" + tip);
    }

    @PostMapping("/course/add")
    public Result addCourse(@RequestBody @Validated AdminAddCourse info, BindingResult result,
                            @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.addCourse(info, token);
    }

    @PostMapping("/course")
    public Result findCourse(@RequestBody @Validated QueryCourseRequest info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.findCourse(info);
    }

    @PutMapping("/course")
    public Result modifyCourse(@RequestBody @Validated AdminModifyCourse info, BindingResult result,
                               @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.modifyCourse(info, token);
    }

    @DeleteMapping("/course")
    public Result deleteCourse(@RequestBody @Validated AdminDeleteCourse info, BindingResult result,
                               @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.deleteCourse(info, token);
    }


    @PostMapping("/timetable/add")
    public Result addTimeTable(@RequestBody @Validated AdminAddTimeTable info, BindingResult result,
                               @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.addTimeTable(info, token);
    }

    @PostMapping("/timetable")
    public Result findTimeTable(@RequestBody @Validated QueryTimeTableRequest info, BindingResult result,
                                @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.findTimeTable(info);
    }

    @PutMapping("/timetable")
    @Deprecated
    public Result modifyTimeTable(@RequestBody @Validated AdminModifyTimeTable info, BindingResult result,
                                  @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.modifyTimeTable(info, token);
    }

    @DeleteMapping("/timetable")
    public Result deleteTimeTable(@RequestBody @Validated AdminDeleteTimeTable info, BindingResult result,
                                  @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.deleteTimeTable(info, token);
    }

    @DeleteMapping("/timetable/detail")
    public Result deleteTimeTableByDetail(@RequestBody @Validated AdminDeleteTimeTableByDetail info, BindingResult result,
                                  @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.deleteTimeTable(info, token);
    }


}
