package org.brynhild.graduation.timetable.executor

import jakarta.annotation.Resource
import org.apache.poi.POIXMLDocument
import org.apache.poi.xwpf.usermodel.*
import org.brynhild.graduation.common.constant.CourseConstant
import org.brynhild.graduation.persistence.timetable.entity.Course
import org.brynhild.graduation.transfer.timetable.bo.ClassStatisticInfo
import org.brynhild.graduation.transfer.timetable.bo.CourseStatisticInfo
import org.brynhild.graduation.transfer.timetable.bo.StudentStatisticInfo
import org.openxmlformats.schemas.wordprocessingml.x2006.main.*
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
class WordExecutor {
    private CosJavaRepository cosJavaRepository

    @Autowired
    @Resource
    void setCosJavaRepository(CosJavaRepository cosJavaRepository) {
        this.cosJavaRepository = cosJavaRepository
    }

    static void main(String[] args) {

    }

    private static final String CLASS_STATISTIC_TEMPLATE2 = "docTemplate/class2.docx"



    private static final String CLASS_STATISTIC_TEMPLATE = "docTemplate/class.docx"

    static void generateClassStatisticReport(String outputFilename, List<ClassStatisticInfo> infoList, Map<String, String> replaceMap) {
        XWPFDocument document = new XWPFDocument(POIXMLDocument.openPackage(CLASS_STATISTIC_TEMPLATE))
        if (replaceMap != null) {
            replaceMarkedText(document, replaceMap)
        }
        def tables = document.getTables()
        def table = tables[0]
        initForm(table)
        int firstIndex = table.getRows().size()
        for (int i = 0; i < infoList.size(); i++) {
            table.createRow()
        }
        for (int i = 0; i < infoList.size(); i++) {
            def cells = table.getRow(firstIndex + i).getTableCells()
            cells[0].text = "${infoList[i].course.id}"
            cells[1].text = infoList[i].course.name
            if (infoList[i].course.type == CourseConstant.COMPULSORY) {
                cells[2].text = "必修"
            } else {
                cells[2].text = "选修"
            }
            cells[3].text = "${infoList[i].course.credit}"
            cells[4].text = "${infoList[i].count}"
            cells[5].text = "${infoList[i].actualCount}"
            cells[6].text = "${infoList[i].rate * 100}%"
            for (cell in cells) {
                setCellCenter(cell)
            }
        }
        File file = new File(outputFilename)
        FileOutputStream fos = new FileOutputStream(file)
        document.write(fos)
        fos.close()
    }

    static void generateStudentStatisticReport(String outputFilename, List<StudentStatisticInfo> infoList, Map<String, String> replaceMap) {
        XWPFDocument document = new XWPFDocument(POIXMLDocument.openPackage(CLASS_STATISTIC_TEMPLATE))
        if (replaceMap != null) {
            replaceMarkedText(document, replaceMap)
        }
        def tables = document.getTables()
        def table = tables[0]
        initForm(table)
        int firstIndex = table.getRows().size()
        for (int i = 0; i < infoList.size(); i++) {
            table.createRow()
        }
        for (int i = 0; i < infoList.size(); i++) {
            def cells = table.getRow(firstIndex + i).getTableCells()
            cells[0].text = "${infoList[i].course.id}"
            cells[1].text = infoList[i].course.name
            if (infoList[i].course.type == CourseConstant.COMPULSORY) {
                cells[2].text = "必修"
            } else {
                cells[2].text = "选修"
            }
            cells[3].text = "${infoList[i].course.credit}"
            cells[4].text = "${infoList[i].count}"
            cells[5].text = "${infoList[i].actualCount}"
            cells[6].text = "${infoList[i].rate * 100}%"
            for (cell in cells) {
                setCellCenter(cell)
            }
        }
        File file = new File(outputFilename)
        FileOutputStream fos = new FileOutputStream(file)
        document.write(fos)
        fos.close()
    }

    static void generateTeachingCourseStatisticReport(String outputFilename, List<CourseStatisticInfo> infoList, Map<String, String> replaceMap) {
        XWPFDocument document = new XWPFDocument(POIXMLDocument.openPackage(CLASS_STATISTIC_TEMPLATE))
        if (replaceMap != null) {
            replaceMarkedText(document, replaceMap)
        }
        def tables = document.getTables()
        def table = tables[0]
        initForm(table)
        int firstIndex = table.getRows().size()
        for (int i = 0; i < infoList.size(); i++) {
            table.createRow()
        }
        for (int i = 0; i < infoList.size(); i++) {
            def cells = table.getRow(firstIndex + i).getTableCells()
            cells[0].text = "${infoList[i].course.id}"
            cells[1].text = infoList[i].course.name
            if (infoList[i].course.type == CourseConstant.COMPULSORY) {
                cells[2].text = "必修"
            } else {
                cells[2].text = "选修"
            }
            cells[3].text = "${infoList[i].course.credit}"
            cells[4].text = "${infoList[i].count}"
            cells[5].text = "${infoList[i].actualCount}"
            cells[6].text = "${infoList[i].rate * 100}%"
            for (cell in cells) {
                setCellCenter(cell)
            }
        }
        File file = new File(outputFilename)
        FileOutputStream fos = new FileOutputStream(file)
        document.write(fos)
        fos.close()
    }


    private static void replaceMarkedText(XWPFDocument document, Map<String, String> replaceMap) {
        List<XWPFParagraph> paragraphs = document.getParagraphs();
        for (XWPFParagraph paragraph : paragraphs) {
            String text = paragraph.getText();
            if (text.contains("\$")) {
                List<XWPFRun> runs = paragraph.getRuns()
                for (XWPFRun run : runs) {
                    run.setText(replaceText(run.toString(), replaceMap), 0);
                }
            }
        }
    }

    private static String replaceText(String value, Map<String, String> replaceMap) {
        println value
        def keySet = replaceMap.keySet()
        for (key in keySet) {
            def matchKey = "\${${key}}"
            if (value.indexOf(matchKey) != -1) {
                return replaceMap[key]
            }
        }
        return value
    }

    private static void setCellCenter(XWPFTableCell cell) {
        CTTc cttc = cell.getCTTc();
        CTTcPr ctpr = cttc.addNewTcPr();
        ctpr.addNewVAlign().setVal(STVerticalJc.CENTER);
        cttc.getPList().get(0).addNewPPr().addNewJc().setVal(STJc.CENTER);
    }

    private static void initForm(XWPFTable table) {
        CTTblBorders borders = table.getCTTbl().getTblPr().addNewTblBorders();
        CTBorder hBorder = borders.addNewInsideH();
        hBorder.setVal(STBorder.Enum.forString("single"));  // 线条类型
        hBorder.setSz(new BigInteger("1")); // 线条大小
        hBorder.setColor("000000"); // 设置颜色

        CTBorder vBorder = borders.addNewInsideV();
        vBorder.setVal(STBorder.Enum.forString("single"));
        vBorder.setSz(new BigInteger("1"));
        vBorder.setColor("000000");

        CTBorder lBorder = borders.addNewLeft();
        lBorder.setVal(STBorder.Enum.forString("single"));
        lBorder.setSz(new BigInteger("1"));
        lBorder.setColor("000000");

        CTBorder rBorder = borders.addNewRight();
        rBorder.setVal(STBorder.Enum.forString("single"));
        rBorder.setSz(new BigInteger("1"));
        rBorder.setColor("000000");

        CTBorder tBorder = borders.addNewTop();
        tBorder.setVal(STBorder.Enum.forString("single"));
        tBorder.setSz(new BigInteger("1"));
        tBorder.setColor("000000");

        CTBorder bBorder = borders.addNewBottom();
        bBorder.setVal(STBorder.Enum.forString("single"));
        bBorder.setSz(new BigInteger("1"));
        bBorder.setColor("000000");
    }
}
