package org.brynhild.graduation.transfer.user.vo;

import jakarta.persistence.criteria.Predicate;
import lombok.Data;
import org.brynhild.graduation.common.tag.SpecificationGenerated;
import org.brynhild.graduation.common.tag.SpecificationIgnore;
import org.brynhild.graduation.common.utils.StringUtil;
import org.brynhild.graduation.persistence.user.entity.User;
import org.springframework.data.jpa.domain.Specification;

import javax.validation.constraints.Email;
import javax.validation.constraints.Positive;
import java.util.ArrayList;
import java.util.List;

@Data
@SpecificationGenerated(User.class)
public class QueryUserRequest {
    @Positive
    @SpecificationIgnore
    private Long id;
    private String username;
    @Email
    private String email;
    private Integer state;
    private String name;
    private String tel;
    private String no;
    private Long departmentId;

    public Specification<User> getSpecification() {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (!StringUtil.isEmpty(username)) {
                predicateList.add(criteriaBuilder.like(root.get("username").as(String.class), "%" + username + "%"));
            }
            if (!StringUtil.isEmpty(email)) {
                predicateList.add(criteriaBuilder.like(root.get("email").as(String.class), "%" + email + "%"));
            }
            if (state != null) {
                predicateList.add(criteriaBuilder.equal(root.get("state").as(Integer.class), state));
            }
            if (!StringUtil.isEmpty(name)) {
                predicateList.add(criteriaBuilder.like(root.get("name").as(String.class), "%" + name + "%"));
            }
            if (!StringUtil.isEmpty(tel)) {
                predicateList.add(criteriaBuilder.like(root.get("tel").as(String.class), "%" + tel + "%"));
            }
            if (!StringUtil.isEmpty(no)) {
                predicateList.add(criteriaBuilder.like(root.get("no").as(String.class), "%" + no + "%"));
            }
            if (departmentId != null) {
                predicateList.add(criteriaBuilder.equal(root.get("departmentId").as(Long.class), departmentId));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }
}
