package org.brynhild.graduation.transfer.view

import jakarta.annotation.Resource
import org.brynhild.graduation.persistence.device.repository.AreaRepository
import org.brynhild.graduation.persistence.timetable.entity.TimeTable
import org.brynhild.graduation.persistence.timetable.repository.CourseRepository
import org.brynhild.graduation.persistence.user.repository.UserRepository
import org.brynhild.graduation.transfer.device.bo.TimeTableBinary
import org.brynhild.graduation.transfer.timetable.dto.TimeTableInfo
import org.springframework.beans.factory.annotation.Autowired

//@Component
class ViewConvertor {
    private UserRepository userRepository
    private AreaRepository areaRepository
    private CourseRepository courseRepository

    TimeTableInfo convertToTimeTableInfo(TimeTable table) {
        def info = new TimeTableInfo()
        def areaOptional = areaRepository.findById(table.areaId)
        def teacherOptional = userRepository.findById(table.teacher)
        info.with {
            id = table.id
            name = table.courseName
            areaOptional.ifPresent { area -> it.room = area.name }
            teacherOptional.ifPresent { teacher -> it.teacher = teacher.name }
            week = table.week
            start = table.index
            step = table.step
            day = table.day
        }
        return info
    }

    List<TimeTableInfo> convertToTimeTableInfo(List<TimeTable> tableList) {
        List<TimeTableInfo> ret = new ArrayList<>()
        if (tableList == null) {
            return ret
        }
        for (table in tableList) {
            ret.add(convertToTimeTableInfo(table))
        }
        return ret
    }

    List<TimeTableBinary> convertToTimetableBinary(TimeTable table) {
        List<TimeTableBinary> ret = new ArrayList<>()
        table.week.split(",").toList().stream().map {
            Integer.parseInt(it)
        }
                .forEachOrdered {
                    def binary = new TimeTableBinary(table)
                    binary.week = it
                    ret.add(binary)
                }

        ret
    }

    @Autowired
    @Resource
    void setUserRepository(UserRepository userRepository) {
        this.userRepository = userRepository
    }

    @Autowired
    @Resource
    void setAreaRepository(AreaRepository areaRepository) {
        this.areaRepository = areaRepository
    }

    @Autowired
    @Resource
    void setCourseRepository(CourseRepository courseRepository) {
        this.courseRepository = courseRepository
    }


}
