package org.brynhild.graduation.trigger.resolve

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.TriggerConstant
import org.brynhild.graduation.common.utils.RandomCodeUtil
import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.transfer.trigger.BuildTask
import org.brynhild.graduation.trigger.event.BuildEventSender
import org.brynhild.graduation.trigger.transfer.UpdateMessage
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
class CommandResolver {
    private BuildEventSender sender

    void resolve(UpdateMessage message) {
        def pusher = message.pusher
        if (pusher == null) {
            return
        }
        if (StringUtil.isEmpty(pusher.login) || "brynhild" != pusher.login) {
            return
        }
        def commits = message.commits
        Set<String> modules = new HashSet<>()
        for (commit in commits) {
            if (commit.committer == null) {
                continue
            }
            if (StringUtil.isEmpty(commit.committer.username) || "brynhild" != commit.committer.username) {
                continue
            }
            def commitMessage = commit.message
            if (StringUtil.isEmpty(commitMessage)) {
                continue
            }
            if (commitMessage.contains(TriggerConstant.BUILD_ALL)) {
                modules.addAll(getAllModules())
                break
            } else if (commitMessage.contains(TriggerConstant.BUILD_TARGET)) {
                modules.addAll(getEffectedModules(commitMessage))

            } else if (commitMessage.contains(TriggerConstant.BUILD_RELATIVE)) {
                modules.addAll(getEffectedModules(commit.added))
                modules.addAll(getEffectedModules(commit.modified))
                modules.addAll(getEffectedModules(commit.removed))
            }
        }
        if (modules.isEmpty()) {
            return
        }

        println ""
        println "--------------TRIGGER------------"
        println "These module will be rebuild:"
        for (String module : modules) {
            println module
        }
        println "---------------------------------"
        println ""

        def task = new BuildTask()
        task.with {
            it.taskId = RandomCodeUtil.getRandomNumberCode(10)
            it.modules = modules
        }
        println "Generate new Task(id:${task.taskId})"

        sender.sendBuildTask(task)
    }

    static Set<String> getEffectedModules(String effectedFile) {
        Set<String> modules = new HashSet<>()
        for (String module : TriggerConstant.moduleSet) {
            if (effectedFile.contains(module)) {
                modules.add(module)
            }
        }
        modules
    }

    static Set<String> getEffectedModules(List<String> changeList) {
        Set<String> modules = new HashSet<>()
        if (changeList == null || changeList.isEmpty()) {
            return modules
        }
        for (String file : changeList) {
            modules.addAll(getEffectedModules(file))
        }
        modules
    }

    static Set<String> getAllModules() {
        Set<String> module = new HashSet<>()
        module.addAll(TriggerConstant.moduleSet)
        module
    }

    @Autowired
    @Resource
    void setSender(BuildEventSender sender) {
        this.sender = sender
    }
}
