package org.brynhild.graduation.user.event

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.constant.RabbitQueueConstant
import org.brynhild.graduation.common.utils.ByteTransferUtil
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.user.entity.User
import org.brynhild.graduation.transfer.user.bo.UserOfflineEvent
import org.brynhild.graduation.transfer.user.vo.LoginRequest
import org.brynhild.graduation.user.factory.OperationLogFactory
import org.springframework.amqp.rabbit.core.RabbitTemplate
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

@Component
class LogSender {
    private RabbitTemplate rabbitTemplate
    private OperationLogFactory operationLogFactory
    private JwtUtil jwtUtil

    void sendOperationLog(OperationLog log) {
        rabbitTemplate.convertAndSend(RabbitQueueConstant.OPERATION_LOG, ByteTransferUtil.getBytesFromObject(log))
    }

    void sendUserLogin(LoginRequest info) {

    }

    void userOffline(String token) {
        def claims = jwtUtil.parseJWT(token)
        def username = claims.get(AccountConstant.TOKEN_USERNAME, String.class)
        def id = jwtUtil.getId(token)
        def name = claims.get(AccountConstant.TOKEN_NAME, String.class)

        def event = new UserOfflineEvent(username)
        def log = operationLogFactory.userOffline(id, name)
        rabbitTemplate.convertAndSend(RabbitQueueConstant.USER, ByteTransferUtil.getBytesFromObject(event))
        rabbitTemplate.convertAndSend(RabbitQueueConstant.OPERATION_LOG, ByteTransferUtil.getBytesFromObject(log))
    }

    void userRegister(User user) {

    }

    @Autowired
    @Resource
    void setRabbitTemplate(RabbitTemplate rabbitTemplate) {
        this.rabbitTemplate = rabbitTemplate
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }


}
