package org.brynhild.graduation.user.factory

import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.user.entity.Academy
import org.brynhild.graduation.persistence.user.entity.Class
import org.brynhild.graduation.persistence.user.entity.Major
import org.brynhild.graduation.persistence.user.entity.User
import org.brynhild.graduation.transfer.user.vo.*
import org.springframework.stereotype.Component

@Component
class OperationLogFactory {
    private final String ln = "\r\n"

    OperationLog userOffline(Long id, String name) {
        def log = new OperationLog()
        log.operatorId = id
        log.operatorName = name
        log.targetId = id
        log.targetName = name
        log.description = "用户：${name} 执行了下线操作"

        log
    }

    //Generated below
    OperationLog generateLogWithAdminModifyAcademy(AdminModifyAcademy info, Academy target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.no)) {
            builder.append("用户").append("修改了学院编号为： ").append(info.no).append(ln)
            target.no = info.no
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了学院名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.avatar)) {
            builder.append("用户").append("修改了图标为： ").append(info.avatar).append(ln)
            target.avatar = info.avatar
        }
        if (info.liable != null) {
            builder.append("用户").append("修改了负责人用户id为： ").append(info.liable).append(ln)
            target.liable = info.liable
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithRegisterUser(RegisterUser info, User target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.username)) {
            builder.append("用户").append("修改了用户名为： ").append(info.username).append(ln)
            target.username = info.username
        }
        if (!StringUtil.isEmpty(info.password)) {
            builder.append("用户").append("修改了密码为： ").append(info.password).append(ln)
            target.password = info.password
        }
        if (!StringUtil.isEmpty(info.email)) {
            builder.append("用户").append("修改了电子邮件地址为： ").append(info.email).append(ln)
            target.email = info.email
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了姓名为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.tel)) {
            builder.append("用户").append("修改了联系电话为： ").append(info.tel).append(ln)
            target.tel = info.tel
        }
        if (!StringUtil.isEmpty(info.no)) {
            builder.append("用户").append("修改了学号/工号为： ").append(info.no).append(ln)
            target.no = info.no
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminAddAcademy(AdminAddAcademy info, Academy target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.no)) {
            builder.append("用户").append("修改了学院编号为： ").append(info.no).append(ln)
            target.no = info.no
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了学院名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.avatar)) {
            builder.append("用户").append("修改了图标为： ").append(info.avatar).append(ln)
            target.avatar = info.avatar
        }
        if (info.liable != null) {
            builder.append("用户").append("修改了负责人用户id为： ").append(info.liable).append(ln)
            target.liable = info.liable
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminAddClass(AdminAddClass info, Class target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.no)) {
            builder.append("用户").append("修改了班级号为： ").append(info.no).append(ln)
            target.no = info.no
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了班级名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (info.majorId != null) {
            builder.append("用户").append("修改了专业id为： ").append(info.majorId).append(ln)
            target.majorId = info.majorId
        }
        if (info.academyId != null) {
            builder.append("用户").append("修改了学院id为： ").append(info.academyId).append(ln)
            target.academyId = info.academyId
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminAddUser(AdminAddUser info, User target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.username)) {
            builder.append("管理员").append("修改了用户名为： ").append(info.username).append(ln)
            target.username = info.username
        }
        if (!StringUtil.isEmpty(info.password)) {
            builder.append("管理员").append("修改了密码为： ").append(info.password).append(ln)
            target.password = info.password
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("管理员").append("修改了姓名为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.sex)) {
            builder.append("管理员").append("修改了性别为： ").append(info.sex).append(ln)
            target.sex = info.sex
        }
        if (info.departmentId != null) {
            builder.append("管理员").append("修改了部门编号为： ").append(info.departmentId).append(ln)
            target.departmentId = info.departmentId
        }
        if (!StringUtil.isEmpty(info.no)) {
            builder.append("管理员").append("修改了学号/工号为： ").append(info.no).append(ln)
            target.no = info.no
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithModifyUser(ModifyUser info, User target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.password)) {
            builder.append("用户").append("修改了密码为： ").append(info.password).append(ln)
            target.password = info.password
        }
        if (!StringUtil.isEmpty(info.email)) {
            builder.append("用户").append("修改了电子邮件地址为： ").append(info.email).append(ln)
            target.email = info.email
        }
        if (!StringUtil.isEmpty(info.avatar)) {
            builder.append("用户").append("修改了头像链接为： ").append(info.avatar).append(ln)
            target.avatar = info.avatar
        }
        if (!StringUtil.isEmpty(info.tel)) {
            builder.append("用户").append("修改了联系电话为： ").append(info.tel).append(ln)
            target.tel = info.tel
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminAddMajor(AdminAddMajor info, Major target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了专业名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.code)) {
            builder.append("用户").append("修改了专业代号为： ").append(info.code).append(ln)
            target.code = info.code
        }
        if (info.academyId != null) {
            builder.append("用户").append("修改了学院id为： ").append(info.academyId).append(ln)
            target.academyId = info.academyId
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminModifyMajor(AdminModifyMajor info, Major target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了专业名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.code)) {
            builder.append("用户").append("修改了专业代号为： ").append(info.code).append(ln)
            target.code = info.code
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminModifyClass(AdminModifyClass info, Class target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.no)) {
            builder.append("用户").append("修改了班级号为： ").append(info.no).append(ln)
            target.no = info.no
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了班级名称为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (info.majorId != null) {
            builder.append("用户").append("修改了专业id为： ").append(info.majorId).append(ln)
            target.majorId = info.majorId
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }


    OperationLog generateLogWithAdminModifyUser(AdminModifyUser info, User target) {
        def log = new OperationLog()
        def originData = target.toString()
        def builder = new StringBuilder()
        if (!StringUtil.isEmpty(info.password)) {
            builder.append("用户").append("修改了密码为： ").append(info.password).append(ln)
        }
        if (!StringUtil.isEmpty(info.avatar)) {
            builder.append("用户").append("修改了头像链接为： ").append(info.avatar).append(ln)
            target.avatar = info.avatar
        }
        if (!StringUtil.isEmpty(info.email)) {
            builder.append("用户").append("修改了电子邮件地址为： ").append(info.email).append(ln)
            target.email = info.email
        }
        if (info.state != null) {
            builder.append("用户").append("修改了用户状态为： ").append(info.state).append(ln)
            target.state = info.state
        }
        if (info.type != null) {
            builder.append("用户").append("修改了用户类型为： ").append(info.type).append(ln)
            target.type = info.type
        }
        if (!StringUtil.isEmpty(info.name)) {
            builder.append("用户").append("修改了姓名为： ").append(info.name).append(ln)
            target.name = info.name
        }
        if (!StringUtil.isEmpty(info.sex)) {
            builder.append("用户").append("修改了性别为： ").append(info.sex).append(ln)
            target.sex = info.sex
        }
        if (info.departmentId != null) {
            builder.append("用户").append("修改了部门编号为： ").append(info.departmentId).append(ln)
            target.departmentId = info.departmentId
        }
        if (!StringUtil.isEmpty(info.tel)) {
            builder.append("用户").append("修改了联系电话为： ").append(info.tel).append(ln)
            target.tel = info.tel
        }
        def newData = info.toString()
        log.originData = originData
        log.newData = newData
        log.description = builder.toString()
        log
    }
}
