package org.brynhild.graduation.user.factory;

import jakarta.persistence.criteria.Predicate;
import org.brynhild.graduation.common.utils.StringUtil;
import org.brynhild.graduation.persistence.user.entity.Academy;
import org.brynhild.graduation.persistence.user.entity.Class;
import org.brynhild.graduation.persistence.user.entity.Major;
import org.brynhild.graduation.persistence.user.entity.User;
import org.brynhild.graduation.transfer.user.vo.QueryAcademyRequest;
import org.brynhild.graduation.transfer.user.vo.QueryClassRequest;
import org.brynhild.graduation.transfer.user.vo.QueryMajorRequest;
import org.brynhild.graduation.transfer.user.vo.QueryUserRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class SpecificationFactory {


    public Specification<User> getUserSpecification(QueryUserRequest info) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (!StringUtil.isEmpty(info.getUsername())) {
                predicateList.add(criteriaBuilder.like(root.get("username").as(String.class), "%" + info.getUsername() + "%"));
            }
            if (!StringUtil.isEmpty(info.getEmail())) {
                predicateList.add(criteriaBuilder.like(root.get("email").as(String.class), "%" + info.getEmail() + "%"));
            }
            if (info.getState() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("state").as(Integer.class), info.getState()));
            }
            if (!StringUtil.isEmpty(info.getName())) {
                predicateList.add(criteriaBuilder.like(root.get("name").as(String.class), "%" + info.getName() + "%"));
            }
            if (!StringUtil.isEmpty(info.getTel())) {
                predicateList.add(criteriaBuilder.like(root.get("tel").as(String.class), "%" + info.getTel() + "%"));
            }
            if (!StringUtil.isEmpty(info.getNo())) {
                predicateList.add(criteriaBuilder.like(root.get("no").as(String.class), "%" + info.getNo() + "%"));
            }
            if (info.getDepartmentId() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("departmentId").as(Long.class), info.getDepartmentId()));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }


    public Specification<Academy> getAcademySpecification(QueryAcademyRequest info) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (!StringUtil.isEmpty(info.getNo())) {
                predicateList.add(criteriaBuilder.like(root.get("no").as(String.class), "%" + info.getNo() + "%"));
            }
            if (!StringUtil.isEmpty(info.getName())) {
                predicateList.add(criteriaBuilder.like(root.get("name").as(String.class), "%" + info.getName() + "%"));
            }
            if (info.getLiable() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("liable").as(Long.class), info.getLiable()));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }

    public Specification<Major> getMajorSpecification(QueryMajorRequest info) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (!StringUtil.isEmpty(info.getName())) {
                predicateList.add(criteriaBuilder.like(root.get("name").as(String.class), "%" + info.getName() + "%"));
            }
            if (!StringUtil.isEmpty(info.getCode())) {
                predicateList.add(criteriaBuilder.like(root.get("code").as(String.class), "%" + info.getCode() + "%"));
            }
            if (info.getAcademyId() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("academyId").as(Long.class), info.getAcademyId()));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }


    public Specification<Class> getClassSpecification(QueryClassRequest info) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (!StringUtil.isEmpty(info.getNo())) {
                predicateList.add(criteriaBuilder.like(root.get("no").as(String.class), "%" + info.getNo() + "%"));
            }
            if (info.getAcademyId() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("academyId").as(Long.class), info.getAcademyId()));
            }
            if (!StringUtil.isEmpty(info.getName())) {
                predicateList.add(criteriaBuilder.like(root.get("name").as(String.class), "%" + info.getName() + "%"));
            }
            if (info.getMajorId() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("majorId").as(Long.class), info.getMajorId()));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }


}


