package org.brynhild.graduation.timetable.controller;

import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.brynhild.graduation.common.parser.LogGenerator;
import org.brynhild.graduation.common.parser.SpecificationGenerator;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.RequestBodyVerifier;
import org.brynhild.graduation.timetable.service.ICourseService;
import org.brynhild.graduation.transfer.timetable.vo.AdminAddCourse;
import org.brynhild.graduation.transfer.timetable.vo.AdminDeleteCourse;
import org.brynhild.graduation.transfer.timetable.vo.AdminModifyCourse;
import org.brynhild.graduation.transfer.timetable.vo.QueryCourseRequest;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.File;

@RestController
@RequestMapping("/course")
@RequiredArgsConstructor
public class CourseController {
    private final RequestBodyVerifier verifier;
    private final ICourseService courseService;

    @PostConstruct
    public void init() {
        String tip = LogGenerator.generate(new File("logGenerate.groovy"), "org.brynhild.graduation.transfer.timetable") ? "ok" : "error";
        System.out.println("generate log:" + tip);
        tip = SpecificationGenerator.generate(new File("specificationGenerate.groovy"), "org.brynhild.graduation.transfer.timetable") ? "ok" : "error";
        System.out.println("generate specification:" + tip);
    }

    @PostMapping("/add")
    public Result addCourse(@RequestBody @Validated AdminAddCourse info, BindingResult result,
                            @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.addCourse(info, token);
    }

    @PostMapping("/")
    public Result findCourse(@RequestBody @Validated QueryCourseRequest info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.findCourse(info);
    }

    @PutMapping("/")
    public Result modifyCourse(@RequestBody @Validated AdminModifyCourse info, BindingResult result,
                               @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.modifyCourse(info, token);
    }

    @DeleteMapping("/")
    public Result deleteCourse(@RequestBody @Validated AdminDeleteCourse info, BindingResult result,
                               @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return courseService.deleteCourse(info, token);
    }

}
