package org.brynhild.graduation.timetable.controller;

import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.RequestBodyVerifier;
import org.brynhild.graduation.timetable.service.ITimeTableService;
import org.brynhild.graduation.transfer.timetable.vo.AdminAddTimeTable;
import org.brynhild.graduation.transfer.timetable.vo.AdminDeleteTimeTable;
import org.brynhild.graduation.transfer.timetable.vo.AdminModifyTimeTable;
import org.brynhild.graduation.transfer.timetable.vo.QueryTimeTableRequest;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/timetable")
@RequiredArgsConstructor
public class TimeTableController {
    private final RequestBodyVerifier verifier;
    private final ITimeTableService tableService;

    @PostMapping("/add")
    public Result addTimeTable(@RequestBody @Validated AdminAddTimeTable info, BindingResult result,
                               @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.addTimeTable(info, token);
    }

    @PostMapping("/")
    public Result findTimeTable(@RequestBody @Validated QueryTimeTableRequest info, BindingResult result,
                                @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.findTimeTable(info);
    }

    @PutMapping("/")
    public Result modifyTimeTable(@RequestBody @Validated AdminModifyTimeTable info, BindingResult result,
                                  @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.modifyTimeTable(info, token);
    }

    @DeleteMapping("/")
    public Result deleteTimeTable(@RequestBody @Validated AdminDeleteTimeTable info, BindingResult result,
                                  @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return tableService.deleteTimeTable(info, token);
    }

}
