package org.brynhild.graduation.timetable.factory;

import jakarta.persistence.criteria.Predicate;
import org.brynhild.graduation.common.utils.StringUtil;
import org.brynhild.graduation.persistence.timetable.entity.Course;
import org.brynhild.graduation.persistence.timetable.entity.TimeTable;
import org.brynhild.graduation.transfer.timetable.vo.QueryCourseRequest;
import org.brynhild.graduation.transfer.timetable.vo.QueryTimeTableRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class SpecificationFactory {
    public Specification<TimeTable> getTimeTableSpecification(QueryTimeTableRequest info) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (info.getAreaId() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("areaId").as(Long.class), info.getAreaId()));
            }
            if (!StringUtil.isEmpty(info.getIndex())) {
                predicateList.add(criteriaBuilder.like(root.get("index").as(String.class), "%" + info.getIndex() + "%"));
            }
            if (!StringUtil.isEmpty(info.getWeek())) {
                predicateList.add(criteriaBuilder.like(root.get("week").as(String.class), "%" + info.getWeek() + "%"));
            }
            if (!StringUtil.isEmpty(info.getClazz())) {
                predicateList.add(criteriaBuilder.like(root.get("clazz").as(String.class), "%" + info.getClazz() + "%"));
            }
            if (info.getCourseId() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("courseId").as(Long.class), info.getCourseId()));
            }
            if (info.getYear() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("year").as(Integer.class), info.getYear()));
            }
            if (info.getTerm() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("term").as(Integer.class), info.getTerm()));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }


    public Specification<Course> getCourseSpecification(QueryCourseRequest info) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (!StringUtil.isEmpty(info.getName())) {
                predicateList.add(criteriaBuilder.like(root.get("name").as(String.class), "%" + info.getName() + "%"));
            }
            if (info.getCredit() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("credit").as(Double.class), info.getCredit()));
            }
            if (info.getDepartment() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("department").as(Long.class), info.getDepartment()));
            }
            if (info.getType() != null) {
                predicateList.add(criteriaBuilder.equal(root.get("type").as(Integer.class), info.getType()));
            }
            if (predicateList.isEmpty()) {
                return null;
            }
            Predicate predicate = predicateList.get(0);
            for (int i = 1; i < predicateList.size(); i++) {
                predicate = criteriaBuilder.and(predicate, predicateList.get(i));
            }
            return predicate;
        };
    }

}
