package org.brynhild.graduation.timetable.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.persistence.device.repository.AreaRepository
import org.brynhild.graduation.persistence.log.entity.OperationLog
import org.brynhild.graduation.persistence.timetable.entity.Course
import org.brynhild.graduation.persistence.timetable.repository.CourseRepository
import org.brynhild.graduation.persistence.timetable.repository.TimeTableRepository
import org.brynhild.graduation.timetable.event.LogSender
import org.brynhild.graduation.timetable.factory.OperationLogFactory
import org.brynhild.graduation.timetable.factory.SpecificationFactory
import org.brynhild.graduation.timetable.service.ITimeTableService
import org.brynhild.graduation.transfer.timetable.vo.AdminAddTimeTable
import org.brynhild.graduation.transfer.timetable.vo.AdminDeleteTimeTable
import org.brynhild.graduation.transfer.timetable.vo.AdminModifyTimeTable
import org.brynhild.graduation.transfer.timetable.vo.QueryTimeTableRequest
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

@Service
class TimeTableServiceImpl implements ITimeTableService {
    private LogSender logSender
    private JwtUtil jwtUtil
    private CourseRepository courseRepository
    private TimeTableRepository tableRepository
    private AreaRepository areaRepository
    private SpecificationFactory specificationFactory
    private OperationLogFactory operationLogFactory

    @Override
    Result addTimeTable(AdminAddTimeTable info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.getId())
        def name = claims.get(AccountConstant.TOKEN_NAME)
        def areaOptional = areaRepository.findById(info.areaId)
        def courseOptional = courseRepository.findById(info.courseId)
        if (areaOptional.isEmpty() || courseOptional.isEmpty()) {
            return new Result(false, "课程或教室不存在")
        }


        def optional = academyRepository.findById(info.department)
        if (optional.isEmpty()) {
            return new Result(false, "目标院校不存在")
        }
        def course = new Course()
        def log = operationLogFactory.generateLogWithAdminAddCourse(info, course)
        courseRepository.save(course)
        log.operatorName = name
        log.operatorId = id
        log.targetName = info.name
        log.targetId = course.id
        logSender.sendOperationLog(log)

        return new Result(true, "添加成功")
    }

    @Override
    Result findTimeTable(QueryTimeTableRequest info) {
        return null
    }

    @Override
    Result modifyTimeTable(AdminModifyTimeTable info, String token) {
        return null
    }

    @Override
    Result deleteTimeTable(AdminDeleteTimeTable info, String token) {
        def claims = jwtUtil.parseJWT(token)
        def id = Long.parseLong(claims.getId())
        def name = claims.get(AccountConstant.TOKEN_NAME)

        def optional = tableRepository.findById(info.id)
        if (optional.isEmpty()) {
            return new Result(false, "删除失败，无相关数据")
        }
        def timeTable = optional.get()
        def log = new OperationLog()
        log.operatorName = name
        log.operatorId = id
        log.targetName = timeTable.courseName
        log.targetId = timeTable.id
        log.description = "管理员${id}->${name}删除了课程安排${timeTable.id}->${timeTable.courseName}"
        courseRepository.logicDelete(timeTable.id)
        logSender.sendOperationLog(log)

        return new Result(true, "删除成功")
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setCourseRepository(CourseRepository courseRepository) {
        this.courseRepository = courseRepository
    }

    @Autowired
    @Resource
    void setTableRepository(TimeTableRepository tableRepository) {
        this.tableRepository = tableRepository
    }

    @Autowired
    @Resource
    void setSpecificationFactory(SpecificationFactory specificationFactory) {
        this.specificationFactory = specificationFactory
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }
}
