package org.brynhild.graduation.transfer.device.bo;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.brynhild.graduation.common.utils.ByteTransferUtil;
import org.brynhild.graduation.common.utils.StringUtil;
import org.brynhild.graduation.persistence.device.entity.Device;

@Data
@NoArgsConstructor
public class NetworkConfig {
    private Integer deviceType;
    private String apSsid;
    private String apPassword;
    private String staSsid;
    private String staPassword;

    public NetworkConfig(Device device) {
        deviceType = device.getType();
        apSsid = device.getApSsid();
        apPassword = device.getApPassword();
        staSsid = device.getStaSsid();
        staPassword = device.getStaPassword();
    }

    public byte[] convertToBytes() {
        byte[] buffer = new byte[84];
        System.arraycopy(ByteTransferUtil.int2Byte(deviceType, true), 0, buffer, 0, 4);
        if (!StringUtil.isEmpty(apSsid)) {
            byte[] apSsidBytes = apSsid.getBytes();
            System.arraycopy(apSsidBytes, 0, buffer, 4, apSsidBytes.length < 20 ? apSsidBytes.length : 19);
        }
        if (!StringUtil.isEmpty(apPassword)) {
            byte[] apPasswordBytes = apPassword.getBytes();
            System.arraycopy(apPasswordBytes, 0, buffer, 24, apPasswordBytes.length < 20 ? apPasswordBytes.length : 19);
        }
        if (!StringUtil.isEmpty(staSsid)) {
            byte[] staSsidBytes = staSsid.getBytes();
            System.arraycopy(staSsidBytes, 0, buffer, 44, staSsidBytes.length < 20 ? staSsidBytes.length : 19);
        }
        if (!StringUtil.isEmpty(staPassword)) {
            byte[] staPasswordBytes = staPassword.getBytes();
            System.arraycopy(staPasswordBytes, 0, buffer, 64, staPasswordBytes.length < 20 ? staPasswordBytes.length : 19);
        }
        return buffer;
    }
}
