package org.brynhild.graduation.user.controller;

import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.RequestBodyVerifier;
import org.brynhild.graduation.transfer.user.vo.*;
import org.brynhild.graduation.user.service.IAdminService;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/admin")
@RequiredArgsConstructor
public class AdminController {
    private final IAdminService adminService;
    private final RequestBodyVerifier verifier;

    @PostMapping("/user")
    public Result findUser(@RequestBody @Validated QueryUserRequest info, BindingResult result,
                           @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.findUser(info, token);
    }

    @PostMapping("/user/add")
    public Result addUser(@RequestBody @Validated AdminAddUser info, BindingResult result,
                          @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        System.out.println(token);
        return adminService.addUser(info, token);
    }

    @PutMapping("/user")
    public Result modifyUser(@RequestBody @Validated AdminModifyUser info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.modifyUser(info, token);
    }

    @DeleteMapping("/user")
    public Result deleteUser(@RequestBody @Validated AdminDeleteUser info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.deleteUser(info, token);
    }

    @PostMapping("/academy")
    public Result findAcademy(@RequestBody @Validated QueryAcademyRequest info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.findAcademy(info);
    }

    @PostMapping("/academy/add")
    public Result addAcademy(@RequestBody @Validated AdminAddAcademy info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.addAcademy(info, token);
    }

    @PutMapping("/academy")
    public Result modifyAcademy(@RequestBody @Validated AdminModifyAcademy info, BindingResult result,
                                @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.modifyAcademy(info, token);
    }

    @DeleteMapping("/academy")
    public Result deleteAcademy(@RequestBody @Validated AdminDeleteAcademy info, BindingResult result,
                                @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.deleteAcademy(info, token);
    }


    @PostMapping("/major")
    public Result findMajor(@RequestBody @Validated QueryMajorRequest info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.findMajor(info);
    }

    @PostMapping("/major/add")
    public Result addMajor(@RequestBody @Validated AdminAddMajor info, BindingResult result,
                           @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.addMajor(info, token);
    }

    @PutMapping("/major")
    public Result modifyMajor(@RequestBody @Validated AdminModifyMajor info, BindingResult result,
                              @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.modifyMajor(info, token);
    }

    @DeleteMapping("/major")
    public Result deleteMajor(@RequestBody @Validated AdminDeleteMajor info, BindingResult result,
                              @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.deleteMajor(info, token);
    }


    @PostMapping("/class")
    public Result findClass(@RequestBody @Validated QueryClassRequest info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.findClass(info);
    }

    @PostMapping("/class/add")
    public Result addClass(@RequestBody @Validated AdminAddClass info, BindingResult result,
                           @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.addClass(info, token);
    }

    @PutMapping("/class")
    public Result modifyClass(@RequestBody @Validated AdminModifyClass info, BindingResult result,
                              @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.modifyClass(info, token);
    }

    @DeleteMapping("/class")
    public Result deleteClass(@RequestBody @Validated AdminDeleteClass info, BindingResult result,
                              @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return adminService.deleteClass(info, token);
    }
}
