package org.brynhild.graduation.user.controller;

import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.constant.AccountConstant;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.RequestBodyVerifier;
import org.brynhild.graduation.transfer.user.vo.ModifyUser;
import org.brynhild.graduation.user.service.ICommonService;
import org.brynhild.graduation.user.service.IDeviceService;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/common")
@RequiredArgsConstructor
public class CommonController {
    private final ICommonService commonService;

    private final IDeviceService deviceService;

    private final RequestBodyVerifier verifier;

    @GetMapping("/offline")
    public Result offline(@RequestHeader(AccountConstant.ACCOUNT_HEADER) String header) {
        return commonService.offline(header);
    }

    @PutMapping("/info")
    public Result modifyInfo(@RequestBody @Validated ModifyUser info, BindingResult result,
                             @RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return commonService.modifyInfo(info, token);
    }

    @PostMapping("/info")
    public Result queryInfo(@RequestHeader(AccountConstant.ACCOUNT_HEADER) String token) {
        System.out.println(token);
        return commonService.queryInfo(token);
    }
}
