package org.brynhild.graduation.user.controller;

import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import org.brynhild.graduation.common.parser.LogGenerator;
import org.brynhild.graduation.common.parser.SpecificationGenerator;
import org.brynhild.graduation.common.tag.Passed;
import org.brynhild.graduation.common.transfer.dto.Result;
import org.brynhild.graduation.common.utils.RequestBodyVerifier;
import org.brynhild.graduation.transfer.user.vo.*;
import org.brynhild.graduation.user.service.IUserService;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;

@RestController
@RequestMapping("/public")
@RequiredArgsConstructor
public class MainController {

    private final RequestBodyVerifier verifier;

    private final IUserService userService;

    @PostConstruct
    public void init() {
        String tip = LogGenerator.generate(new File("logGenerate.groovy"), "org.brynhild.graduation.transfer.user") ? "ok" : "error";
        System.out.println("generate log:" + tip);
        tip = SpecificationGenerator.generate(new File("specificationGenerate.groovy"), "org.brynhild.graduation.transfer.user") ? "ok" : "error";
        System.out.println("generate specification:" + tip);
    }

    @Passed
    @PostMapping("/register")
    public Result register(@RequestBody @Validated RegisterUser info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return userService.register(info);
    }

    @PostMapping("/active")
    public Result active(@RequestBody @Validated ActiveAccount hash, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return userService.active(hash.getHash());
    }

    @PostMapping("/reset")
    public Result reset(@RequestBody @Validated ResetPassword info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return userService.reset(info);
    }

    @Passed
    @PostMapping("/resend")
    public Result resend(@RequestBody @Validated ResendEmail info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return userService.resend(info);
    }

    @Passed
    @PostMapping("/login")
    public Result login(@RequestBody @Validated LoginRequest info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return userService.login(info);
    }

    @Passed
    @PostMapping("/token/valid")
    public Result tokenValid(@RequestBody @Validated ValidToken info, BindingResult result) {
        if (result.hasErrors()) {
            return verifier.convertToErrors(result);
        }
        return userService.tokenValid(info.getToken());
    }
}
