package org.brynhild.graduation.user.service.impl

import jakarta.annotation.Resource
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.constant.UserStateConstant
import org.brynhild.graduation.common.transfer.dto.Result
import org.brynhild.graduation.common.utils.JwtUtil
import org.brynhild.graduation.common.utils.StringUtil
import org.brynhild.graduation.persistence.user.entity.User
import org.brynhild.graduation.persistence.user.repository.UserRepository
import org.brynhild.graduation.transfer.user.vo.ModifyUser
import org.brynhild.graduation.user.event.LogSender
import org.brynhild.graduation.user.factory.OperationLogFactory
import org.brynhild.graduation.user.service.ICommonService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.stereotype.Service

@Service
class CommonServiceImpl implements ICommonService {
    private JwtUtil jwtUtil
    private LogSender logSender
    private BCryptPasswordEncoder encoder
    private UserRepository userRepository
    private OperationLogFactory operationLogFactory

    @Override
    Result offline(String token) {
        logSender.userOffline(token)
        return new Result(true, "已通知下线")
    }

    @Override
    Result modifyInfo(ModifyUser info, String token) {
        def user = findUserByToken(token)
        if (user.isEmpty()) {
            return new Result(false, "用户不存在")
        }
        user = user.get()
        def log = operationLogFactory.generateLogWithModifyUser(info, user)
        if (!StringUtil.isEmpty(info.password)) {
            user.password = encoder.encode(info.password)
        }
        if (!StringUtil.isEmpty(info.email) && info.email != user.email) {
            user.state = UserStateConstant.NOT_ACTIVE
        }
        log.operatorId = log.targetId = user.id
        log.operatorName = log.targetName = user.name

        logSender.sendOperationLog(log)
        userRepository.save(user)
        return new Result(true, "修改成功")
    }

    @Override
    Result queryInfo(String token) {
        def user = findUserByToken(token)
        if (user.isEmpty()) {
            return new Result(false, "用户不存在")
        }
        user = user.get()
        return new Result(true, "查询成功", user)
    }

    private Optional<User> findUserByToken(String token) {
        return userRepository.findById(Long.parseLong(jwtUtil.parseJWT(token).get(AccountConstant.TOKEN_ID) as String))
    }

    @Autowired
    @Resource
    void setJwtUtil(JwtUtil jwtUtil) {
        this.jwtUtil = jwtUtil
    }

    @Autowired
    @Resource
    void setLogSender(LogSender logSender) {
        this.logSender = logSender
    }

    @Autowired
    @Resource
    void setOperationLogFactory(OperationLogFactory operationLogFactory) {
        this.operationLogFactory = operationLogFactory
    }

    @Autowired
    @Resource
    void setEncoder(BCryptPasswordEncoder encoder) {
        this.encoder = encoder
    }

    @Autowired
    @Resource
    void setUserRepository(UserRepository userRepository) {
        this.userRepository = userRepository
    }
}
